package gridcomp.adl;

import java.util.List;
import java.util.Map;

import org.objectweb.deployment.scheduling.component.api.InstanceProviderTask;
import org.objectweb.deployment.scheduling.component.lib.AbstractConfigurationTask;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.TaskMap;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.components.PrimitiveCompiler;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

public class BeskeCompiler implements PrimitiveCompiler, BindingController
{
	public final static String BUILDER_FARM = "builder";
	private BeskeBuilder builder;
	
    @SuppressWarnings("unchecked")
	public void compile(List path, ComponentContainer container, TaskMap tasks, Map context)  throws ADLException
    {
    	if (container instanceof BeskeContainer) {
    		Beske f = ((BeskeContainer)container).getBeske();
    		if (f != null) {
    			InstanceProviderTask c = (InstanceProviderTask)tasks.getTask("create", container);
    			SetBeskeAttributesTask t1 = new SetBeskeAttributesTask(builder, f.getXmlFile());
    			ActivateMonitorTask t2 = new ActivateMonitorTask(builder);
    			t1.setInstanceProviderTask(c);
    			t2.setInstanceProviderTask(c);
    			tasks.addTask("beske",   container, t1);
    			tasks.addTask("monitor", container, t2);
    		}
    	}
    	if (container instanceof ComponentContainer) {
			InstanceProviderTask c = (InstanceProviderTask)tasks.getTask("create", container);
			ActivateMonitorTask t2 = new ActivateMonitorTask(builder);
			t2.setInstanceProviderTask(c);
			tasks.addTask("monitor", container, t2);    		
    	}
    }
    
    static class SetBeskeAttributesTask extends AbstractConfigurationTask 
    {
    	private BeskeBuilder builder;
    	private String xmlFile;
    	
    	public SetBeskeAttributesTask(BeskeBuilder builder, String xmlFile)
    	{
    		this.builder = builder;
    		this.xmlFile = xmlFile;
    	}
    	
    	public void execute(final Object context) throws Exception
    	{
    		Object component = getInstanceProviderTask().getResult();
    		builder.setBeskeAttributes(component, xmlFile);
    	}

		public Object getResult() 
		{
			return null;
		}

		public void setResult(Object arg0) 
		{
		}
    }

    static class ActivateMonitorTask extends AbstractConfigurationTask 
    {
    	private BeskeBuilder builder;
    	
    	public ActivateMonitorTask(BeskeBuilder builder)
    	{
    		this.builder = builder;
    	}
    	
    	public void execute(final Object context) throws Exception
    	{
    		Object component = getInstanceProviderTask().getResult();
    		builder.activateMonitor(component);
    	}

		public Object getResult() 
		{
			return null;
		}

		public void setResult(Object arg0) 
		{
		}
    }

    // BindingController methods
    
    public void bindFc(String itf, Object value) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException
    {
        if (itf.equals(BUILDER_FARM)) {
            builder = (BeskeBuilder) value;
            return;
        }
        throw new NoSuchInterfaceException("Cannot find interface " + itf);
    }

    public String[] listFc()
    {
        return new String[]{BUILDER_FARM};
    }

    public Object lookupFc(String itf) throws NoSuchInterfaceException
    {
        if (itf.equals(BUILDER_FARM))
            return builder;
        throw new NoSuchInterfaceException("Cannot find interface " + itf);
    }

    public void unbindFc(String itf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException
    {
        if (itf.equals(BUILDER_FARM)) {
            builder = null;
            return;
        }
        throw new NoSuchInterfaceException("Cannot find interface " + itf);
    }
}
