package gridcomp.controller;

import java.io.File;
import java.util.Hashtable;
import java.util.Map;

import gridcomp.AutonomicException;
import gridcomp.operation.AutonomicOperation;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.proactive.core.ProActiveRuntimeException;
import org.objectweb.proactive.core.component.controller.AbstractProActiveController;
import org.objectweb.proactive.core.component.type.ProActiveTypeFactoryImpl;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;

public abstract class AbstractAutonomicController 
	extends AbstractProActiveController 
	implements AutonomicController 
{
    protected final Map<String, AutonomicOperation> operationMap;
    protected final Map<String, Object> knowledgeMap;

    public AbstractAutonomicController(Component owner) 
    {
        super(owner);
        knowledgeMap = new Hashtable<String, Object>(); 
        operationMap = new Hashtable<String, AutonomicOperation>();
        knowledgeMap.put(KnowledgeConstants.PARENT, owner);
    }
 
    protected void setControllerItfType() 
    {
        try {
            setItfType(ProActiveTypeFactoryImpl.instance().createFcItfType(
                    AutonomicController.CONTROLLER_NAME,
                    AutonomicController.class.getName(),
                    TypeFactory.SERVER,
                    TypeFactory.OPTIONAL,
                    TypeFactory.SINGLE));
        } catch (InstantiationException e) {
            throw new ProActiveRuntimeException("Cannot create controller type: " + getClass().getName());
        }
    }
    
    public final void addAutonomicOperation(String id, AutonomicOperation aop) 
    {
        if (id != null && aop != null)
            operationMap.put(id, aop);
    }

    public final AutonomicOperation getAutonomicOperation(String id) 
    {
        if (id != null)
            return operationMap.get(id);
        return null;
    }

    public final GenericTypeWrapper<Object> execOperation(String op, Object... params) throws AutonomicException
    {
        Object result = null;
        AutonomicOperation aop = operationMap.get(op);
        if (aop == null)
            System.err.println(op + " not supported!");
        else
            result = aop.execute(params);
        return new GenericTypeWrapper<Object>(result);
    }

    public final String[] listAutonomicOperations() throws AutonomicException
    {
        return operationMap.keySet().toArray(new String[operationMap.size()]);
    }
    
    public void autonomicInit(File beske) throws Exception
    {
    }
    
    public final Map <String, Object> getKnowledge()
    {
    	return this.knowledgeMap;
    }
}
