package gridcomp.test;

import static org.junit.Assert.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.proactive.api.PADeployment;
import org.objectweb.proactive.core.body.request.Request;
import org.objectweb.proactive.core.component.Constants;
import org.objectweb.proactive.core.component.controller.PriorityController;
import org.objectweb.proactive.core.component.controller.QueueController;
import org.objectweb.proactive.core.component.controller.PriorityController.RequestPriority;
import org.objectweb.proactive.core.component.request.ComponentRequest;
import org.objectweb.proactive.core.descriptor.data.ProActiveDescriptor;
import org.objectweb.proactive.core.util.wrapper.IntWrapper;

public class QueueControllerTest extends GridcompTest
{
    private Component root = null;
    
    private Component c1 = null, c2 = null;
    private I fi1 = null, fi2  = null;
    private QueueController qc1 = null, qc2 = null;

    @Before
    public void createComponents() throws Exception
    {
        PriorityController pc = null;
        
        ProActiveDescriptor pad = PADeployment.getProactiveDescriptor("local-deployment.xml");
        Map<String, Object> context = new HashMap<String, Object>(1);
        context.put("deployment-descriptor", pad);

        Factory fact = gridcomp.adl.FactoryFactory.getFactory();
        root =  (Component) fact.newComponent("gridcomp.test.adl.QueueControllerTest", context);
        
        assertNotNull(root);
        
        // start component (to init everything)
        Fractal.getLifeCycleController(root).startFc();
        // stop component (we check everything with stopped components)
        Fractal.getLifeCycleController(root).stopFc();
        
        ContentController cc = Fractal.getContentController(root);
        Component[] comps = cc.getFcSubComponents();
        for (Component c: comps) {
            if (Fractal.getNameController(c).getFcName().equals("dummy1"))
                c1 = c;
            else
                c2 = c;
        }
        assertNotNull(c1);
        assertNotNull(c2);

        fi1 = (I)c1.getFcInterface(I.ITF_SERVER_NAME);
        fi2 = (I)c2.getFcInterface(I.ITF_SERVER_NAME);
        assertNotNull(fi1);
        assertNotNull(fi2);
        
        qc1 = (QueueController)c1.getFcInterface(Constants.REQUEST_QUEUE_CONTROLLER);
        qc2 = (QueueController)c2.getFcInterface(Constants.REQUEST_QUEUE_CONTROLLER);
        assertNotNull(qc1);
        assertNotNull(qc2);
                
        pc = (PriorityController) c1.getFcInterface(Constants.REQUEST_PRIORITY_CONTROLLER);
        assertNotNull(pc);
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "getSize",      RequestPriority.NF2);
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "popRequest",   RequestPriority.NF2);
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "pushRequest",  RequestPriority.NF2);        
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "popRequests",  RequestPriority.NF2);
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "pushRequests", RequestPriority.NF2);
        
        pc = (PriorityController) c2.getFcInterface(Constants.REQUEST_PRIORITY_CONTROLLER);
        assertNotNull(pc);
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "getSize",      RequestPriority.NF2);
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "popRequest",   RequestPriority.NF2);
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "pushRequest",  RequestPriority.NF2);
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "popRequests",  RequestPriority.NF2);
        pc.setPriority(Constants.REQUEST_QUEUE_CONTROLLER, "pushRequests", RequestPriority.NF2);

    }

    @Test
    public void test_getSize_1() throws Exception
    {
        IntWrapper size;
        int n1;

        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1, 0);
        fi1.voidCall();
        fi1.voidCall();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,2);
        fi1.voidCall();
        fi1.voidCall();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,4);
    }  
    
    @Test
    public void test_getSize_2() throws Exception
    {
        IntWrapper size;
        int n2;

        Fractal.getLifeCycleController(c2).startFc();
        size = qc2.getSize();
        n2 = size.intValue();
        assertEquals(n2, 0);
        for (int i = 0; i < 100; i++)
            fi2.voidCall();
        size = qc2.getSize();
        n2 = size.intValue();
        assertEquals(n2,99);

        for (int i = 0; i < 33; i++)
            fi2.voidCall();
        size = qc2.getSize();
        n2 = size.intValue();
        assertEquals(n2,131);
        Fractal.getLifeCycleController(c2).stopFc();
    }  
    
    @Test
    public void test_popRequest_1() throws Exception
    {
        IntWrapper size;
        int n1;
        for (int i = 0; i < 4; i++)
            fi1.voidCall();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,4);

        Request req = qc1.popRequest();
        req.getTimeStamp();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,3);
        
        for (int i = 0; i < 4; i++)
            fi1.voidCall();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,7);
    }

    @Test
    public void test_popRequest_2() throws Exception
    {
        IntWrapper size;
        int n2;
        
        Fractal.getLifeCycleController(c2).startFc();
        for (int i = 0; i < 4; i++)
            fi2.voidCall();
        size = qc2.getSize();
        n2 = size.intValue();
        assertEquals(n2,3);

        Request req = qc2.popRequest();
        req.getTimeStamp();
        size = qc2.getSize();
        n2 = size.intValue();
        assertEquals(n2,0);

        for (int i = 0; i < 4; i++)
            fi2.voidCall();
        size = qc2.getSize();
        n2 = size.intValue();
        assertEquals(n2,3);
        Fractal.getLifeCycleController(c2).stopFc();
    }
    
    @Test
    public void test_popRequests_1() throws Exception
    {
        IntWrapper size;
        int n1;
        
        for (int i = 0; i < 4; i++)
            fi1.voidCall();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,4);

        List<ComponentRequest> rem = qc1.popRequests(2);
        assertEquals(rem.size(),2);
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,2);

        for (int i = 0; i < 4; i++)
            fi1.voidCall();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,6);
    }

    @Test
    public void test_popRequests_2() throws Exception
    {
        IntWrapper size;
        int n2;
        
        Fractal.getLifeCycleController(c2).startFc();
        for (int i = 0; i < 8; i++)
            fi2.voidCall();
        size = qc2.getSize();
        n2 = size.intValue();
        assertEquals(n2,7);

        List<ComponentRequest> rem = qc2.popRequests(2);
        assertEquals(rem.size(),2);
        size = qc2.getSize();
        n2 = size.intValue();
        assertEquals(n2,3);

        for (int i = 0; i < 8; i++)
            fi2.voidCall();
        size = qc2.getSize();
        n2 = size.intValue();
        assertEquals(n2,10);
        Fractal.getLifeCycleController(c2).stopFc();
    }
 
    @Test
    public void test_pushRequest_1() throws Exception
    {
        IntWrapper size;
        int n1;
        
        for (int i = 0; i < 4; i++)
            fi1.voidCall();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,4);

        ComponentRequest req = qc1.popRequest();
        assertNotNull(req);
        req.getTimeStamp();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,3);

        for (int i = 0; i < 4; i++)
            fi2.voidCall();
        size = qc2.getSize();
        n1 = size.intValue();
        assertEquals(n1,4);

        qc2.pushRequest(req);
        size = qc2.getSize();
        n1 = size.intValue();
        assertEquals(n1,5);
    }

    @Test
    public void test_pushRequest_2() throws Exception
    {
        IntWrapper size;
        int n1;
        
        Fractal.getLifeCycleController(root).startFc();
        
        for (int i = 0; i < 4; i++)
            fi1.voidCall();
        // serving 1
        size = qc1.getSize();
        // serving 2
        n1 = size.intValue();
        assertEquals(n1,3);
        
        ComponentRequest req = qc1.popRequest();
        // serving 3
        assertNotNull(req);
        req.getTimeStamp();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,0);
        
        for (int i = 0; i < 4; i++)
            fi2.voidCall();
        // serving 1
        size = qc2.getSize();
        // serving 2
        n1 = size.intValue();
        assertEquals(n1,3);

        qc2.pushRequest(req);
        size = qc2.getSize();
        // serving 3,4,5
        n1 = size.intValue();
        assertEquals(n1,3);
        
        Fractal.getLifeCycleController(root).stopFc();
    }

    @Test
    public void test_pushRequests_1() throws Exception
    {
        IntWrapper size;
        int n1;
        
        for (int i = 0; i < 8; i++)
            fi1.voidCall();
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,8);

        List<ComponentRequest> list = qc1.popRequests(3);
        assertNotNull(list);
        assertEquals(list.size(),3);
        size = qc1.getSize();
        n1 = size.intValue();
        assertEquals(n1,5);

        
        for (int i = 0; i < 8; i++)
            fi2.voidCall();
        size = qc2.getSize();
        n1 = size.intValue();
        assertEquals(n1,8);
        
        qc2.pushRequests(list);
        size = qc2.getSize();
        n1 = size.intValue();
        assertEquals(n1,11);
    }

    @Test
    public void test_pushRequests_2() throws Exception
    {
        IntWrapper size;
        int n1;
        
        Fractal.getLifeCycleController(root).startFc();
        for (int i = 0; i < 8; i++)
            fi1.voidCall();
        // serving 1
        size = qc1.getSize();
        // serving 2
        n1 = size.intValue();
        assertEquals(n1,7);

        List<ComponentRequest> list = qc1.popRequests(3);
        // serving 3
        assertNotNull(list);
        assertEquals(list.size(),3);
        
        // Thread.sleep(500);
        
        size = qc1.getSize();
        // serving 4,5
        n1 = size.intValue();
        assertEquals(n1,2);

        for (int i = 0; i < 8; i++)
            fi2.voidCall();
        // serving 1
        size = qc2.getSize();
        // serving 2
        n1 = size.intValue();
        assertEquals(n1,7);
        
        qc2.pushRequests(list);
        size = qc2.getSize();
        // serving 3,4,5,6,7,8,9,10,11
        n1 = size.intValue();
        assertEquals(n1,9);
        
        Thread.sleep(15000);
        Fractal.getLifeCycleController(root).stopFc();
    }    
}
