package gridcomp.manager.map.impl;
import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Date;

import gridcomp.manager.util.MyMonitorFormatter;
import java.util.HashMap;
import java.util.logging.Logger;
import gridcomp.manager.beans.CounterBean;
import gridcomp.manager.beans.QoSEvent;
import gridcomp.manager.beans.QoSItemBean;
import gridcomp.manager.operations.ManagerCommand;
import gridcomp.manager.operations.ManagerOperation;
import gridcomp.manager.operations.ManagerSetupCommand;
import gridcomp.map.controller.WorkerNameController;
import gridcomp.map.controller.operation.GetServiceTime;
import gridcomp.map.controller.operation.MapMonitor;
import gridcomp.map.controller.operation.MapNumWorkers;
import gridcomp.manager.operation.*;
import gridcomp.manager.util.RulesParser;
import org.objectweb.proactive.core.util.wrapper.DoubleWrapper;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;
import java.util.logging.FileHandler;

import java.util.logging.Handler;

public class MapAutonomicManager 
 extends gridcomp.manager.AbstractAutonomicManager 
	implements WorkerNameController{
	private String WorkerName;
	private String contract;
	private HashMap<String,Double> methodsToMonitor = new HashMap<String, Double>();
	private String contractFilePath;
	private String monitoringMethodName;
	private boolean update =  false;
	private String managerFileLogger;
	private final Logger logger = Logger.getLogger(MapAutonomicManager.class.getName());
	private String DescriptorFilePath;
	QoSItemBean<Double> bean1 = null;
	QoSItemBean<Double> bean2 = null;
	QoSItemBean<Integer> bean3 = null;
	private QoSItemBean<Long> counterBean = new CounterBean(this,0L);
	private final Logger remote = Logger.getLogger("MONITORING");
	//private File managerMonitorFile = null;
	private String myhost =null;
	private long init=0;
	private Date mydate;
	 	
	
	public GenericTypeWrapper<Object> raiseViolation(GenericTypeWrapper<Object> violationId) {
		System.err.println("[MapAutonomicManager] raiseViolation");

		return null;
	}

	public String getWorkerName() {
		return WorkerName;
	}

	public void setWorkerName(String s) {
		WorkerName = s;
	}
	
	@Override 
	public void initAutonomicManager(){
       /* ManagerOperation is the action that it's given to the user in the contract. The user can use this action 
        * in the fire operation method.
        */  
		logger.info("AutonomicManager: initManager");
		
		addAutomomicOperation(ManagerOperation.ADD_EXECUTOR, new MapAddExecutorCommand(managedAutonomicController,this.WorkerName, this.DescriptorFilePath));
		addAutomomicOperation(ManagerOperation.REMOVE_EXECUTOR, new MapSubExecutorCommand(managedAutonomicController, this.WorkerName));
		addAutomomicOperation(ManagerOperation.SETUP, new ManagerSetupCommand(this));
		try {
			Handler remoteHandler = new FileHandler(managerFileLogger);
			remoteHandler.setFormatter(new MyMonitorFormatter("# wall_clock_time time_from_start partition_size n_workers\n"));
			remote.addHandler(remoteHandler);
		} catch (IOException e) {
			logger.warning("Cannot set manager logger. Maybe ManagerMonitorServer is down!");
		}
		try {
			myhost = InetAddress.getLocalHost().getHostName();
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		logger.info("MapAutonomicManager initialized on "+myhost+"........");
		
		super.initAutonomicManager();
		init = System.currentTimeMillis();
	}

	

	@Override
	public void addMethodToMonitor(String data) {
		if (!methodsToMonitor.containsKey(data))
		{
			methodsToMonitor.put(data, 0.0);
		}
	}

	@Override
	public void removeMethodToMonitor(String data) {
		//System.err.println("[MapAutonomicManager] dentro removemethodtomonitor");	
		methodsToMonitor.remove(data);
	}

	@Override
	public void sendEvent(GenericTypeWrapper<QoSEvent> ev) {
		//logger.info("Starting a sendEvent");
		QoSEvent event = ev.getObject();
		ManagerOperation operation = event.getOperation();
		ManagerCommand cmd = lookupOperation(operation);
		if (operation == ManagerOperation.SETUP)
		{
			cmd.setData(event.getData());
		}
		if (cmd != null)
		{
			logger.info("Executing reconfiguration command");
			cmd.execute();
		} else {
			logger.info("Reconfiguration operation not allowed");
		}
	}

	@Override
	protected void setupQoS() {
		bean3 = createNumWorkersBean();
		long end=0;
		for (String key: methodsToMonitor.keySet())
		{
			logger.info("Setting QoS information for method "+ key);
			bean1 = definePartitionSize(key);
			bean2 = defineServiceTime(key);
			double value = bean1.getValue();
			logger.info("#method:"+key+"#time:"+System.currentTimeMillis()+"#rate:"+value+"#workers:"+bean3.getValue().intValue());
			//remote.info("#method:"+key+"#time:"+System.currentTimeMillis()+"#rate:"+value+"#workers:"+bean3.getValue().intValue());
			end = System.currentTimeMillis();
			mydate = new Date();
			remote.info(mydate.getTime()+"\t"+(end-init)+"\t"+value+"\t"+bean3.getValue().intValue()+"\n");
			addQoSItem(bean1,update);	
			addQoSItem(bean2,update);
			
		}
		//init=end;
//		 just insert it only a time.
		
		addQoSItem(counterBean,update);
		addQoSItem(bean3,update);
			
		update = true;
	}
	private QoSItemBean<Integer> createNumWorkersBean() {
		// valutare la creazione del bean se va bene ad ogni attivazione oppure no
		Integer workers = (Integer)((GenericTypeWrapper) managedAutonomicController.execOperation(MapNumWorkers.OPNAME)).getObject();
		
		QoSItemBean<Integer> bean = new NumWorkerBean(this, workers);
		return bean;
	 }
	private QoSItemBean<Double> defineServiceTime(String key) {
		GenericTypeWrapper<Object> obj = managedAutonomicController.execOperation(GetServiceTime.OPNAME);
		DoubleWrapper ts = (DoubleWrapper)obj.getObject();
    	QoSItemBean<Double> bean = new ServiceTimeBean(this,ts.doubleValue());
		//System.out.println("[MapAutonomicManager] Last Service Time="+ts.doubleValue());
    	return bean;
			
	}
	private QoSItemBean<Double> definePartitionSize(String key) {
		GenericTypeWrapper<Object> obj = managedAutonomicController.execOperation(MapMonitor.OPNAME);
    	DoubleWrapper size = (DoubleWrapper)obj.getObject();
    	QoSItemBean<Double> bean = new PartitionSizeBean(this,size.doubleValue());
		return bean;
			
	}


	public String getMonitoringMethodName() {
		return monitoringMethodName;
	}


	public String getPerformanceContract() {
		return contract;
	}

	public String getPerformanceContractFile() {
		return contractFilePath;
	}


	public void setMonitoringMethodName(String name) {
		monitoringMethodName = name;
	}


	public void setPerformanceContractFile(String fName) {
	File f = new File(fName);
		
		if (f.exists())
		{
			logger.info("Reading contract from file " + fName+" on "+myhost);
			contractFilePath = f.getAbsolutePath();
			RulesParser parser = new RulesParser();
			String parsed = parser.parse(f.getAbsolutePath());
			if ((parsed!= null) && (parsed.length() > 0))
			{
				
				ArrayList<String> methodNames = parser.getMethodsNames();
				for (String method: methodNames)
				{
					
					String tmp = method.replace("\"", "");
					methodsToMonitor.put(tmp, 0.0);
				}
				//contract = parsed;
				super.commitContract(parsed);
			}
		}else
			logger.info("Any QoS contract submitted "+f.getAbsolutePath());
		
	}

	public long getAutonomicLoopCycle() {return 0;}

	public String getLoggerFileName() {
		return this.managerFileLogger;
	}

	public void setAutonomicLoopCycle(long loop) {}

	public void setLoggerFileName(String name) {
		this.managerFileLogger = name;
	}

	public String getDescriptorFilePath() {
		return DescriptorFilePath;
	}

	public void setDescriptorFilePath(String s) {
		DescriptorFilePath =s;
		
	}
	
	
	
	
}



