package gridcomp.map.controller.operation;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.StringTokenizer;
import java.util.Vector;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.proactive.core.descriptor.data.VirtualNode;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.proactive.api.PADeployment;
import org.objectweb.proactive.core.ProActiveException;
import org.objectweb.proactive.core.component.Constants;
import org.objectweb.proactive.core.component.controller.MulticastController;
import org.objectweb.proactive.core.component.group.ProxyForComponentInterfaceGroup;
import org.objectweb.proactive.core.component.identity.ProActiveComponent;
import org.objectweb.proactive.core.descriptor.data.ProActiveDescriptor;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;

//import doublemulticast.controller.MulticastControllerWithRedistribution;
import gridcomp.AutonomicException;
import gridcomp.controller.AutonomicController;
//import gridcomp.controller.ContextConstants;
import gridcomp.controller.KnowledgeConstants;
import gridcomp.operation.AbstractAutonomicOperation;
import gridcomp.map.controller.MapController;
import gridcomp.map.Partition;
import gridcomp.map.Task;

public class MapAddWorker  extends AbstractAutonomicOperation {
	
	//private static final Logger logger = Logger.getLogger(Loggers.OPERATION);
	public static String OPNAME = "MapAddWorker";
	Random generator =null;
	public MapAddWorker(AutonomicController ac) {
		super(ac);
		generator = new Random();
	}

	private Vector<MulticastBinding> getPairInterfaces(MulticastController multiCC, Vector<String> multicastInterfaceNames){
		Interface[] itf=null;
		Vector<MulticastBinding> mb = new Vector<MulticastBinding>();
		if (multiCC == null){
			System.out.println("[MapAddWorker] Null reference to the worker set");
			throw new NullPointerException();
		}
		String multicastItfname;
		itf = new Interface[multicastInterfaceNames.size()];
		for (int i=0;i<multicastInterfaceNames.size();i++){
			multicastItfname = multicastInterfaceNames.get(i);

			ProxyForComponentInterfaceGroup group = multiCC.lookupFcMulticast(multicastItfname);
			
			Iterator it  = group.getDelegatee().iterator();
			Object o = it.next();
		
			if (o==null){
				System.out.println("[MapAddWorker] Null reference to the worker set");
				itf=null;
			}else{
				itf[i] = (Interface) o;
				mb.add(new MulticastBinding(multicastItfname,itf[i]));
			}
		}
		//return itf;
		return mb;
	}
	private Vector<String> getMulticastItfNumber(Component owner) {
		Vector<String> multiItf = new Vector<String>();
		Interface[] itf = (Interface[]) ((ProActiveComponent) owner).getRepresentativeOnThis().getFcInterfaces();
		 int i=0;
		// int ret=0;
		 for (;i<itf.length;i++){
			 String itfname = itf[i].getFcItfName();
			 if  (itfname.startsWith(MapController.MulticastInterfaceName)){
				 multiItf.add(itf[i].getFcItfName());
			}
		  }
		 //System.out.println("Interface found ->"+ret);
		return multiItf;
	}
	@Override
	protected void checkContext() throws AutonomicException {
    	if (owner.getKnowledge().get(KnowledgeConstants.PARENT) == null)
    		throw new AutonomicException("The component has not been set in context!");
    	//logger.info("MapAddWorker::Context successfully checked.");

	}

	@Override
	protected Object exec(Object... params) throws AutonomicException {
		String adl = (String) params[0];
		String padPath = (String) params[1];
		System.out.println("\n========================================================================");
		System.out.println("                         [Start NF-MapAddWorker]                        ");
		System.out.println("========================================================================\n");
		//ProActiveComponent _owner = (ProActiveComponent) ctx.get(ContextConstants.PARENT);
	
		ProActiveComponent _owner = (ProActiveComponent) (owner.getKnowledge().get(KnowledgeConstants.PARENT));
        
		
		/*--------------------------------------- STOP THE COMPONENT -----------------------------*/
		LifeCycleController lcc=null;
		try {
			lcc = (LifeCycleController) _owner.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
			((gridcomp.map.controller.LifeCycleControllerImpl)lcc).autonomicStopFc();
			//((dpskeleton.core.manager.impl.LifeCycleControllerImpl)lcc).autonomicStopFc();
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}
		ContentController cc=null;
		try {
			cc = (ContentController) _owner.getFcInterface("content-controller");
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}
		Component[] oldconfig = cc.getFcSubComponents();
		
		/*----------------------------------------- CREATE THE COMPONENT ------------------------*/
		Component worker=null;
		try {
			worker = newComponent(adl,padPath);
		} catch (ProActiveException e) {
			e.printStackTrace();
		} catch (ADLException e) {
			e.printStackTrace();
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}
		if (worker==null){
			System.out.println("[MapAddWorker] Null worker reference");
			System.exit(-1);
		}
		/*--------------------------------------- ADD THE COMPONENT -------------------------------*/
		
		try {
			cc.addFcSubComponent(worker);
		} catch (IllegalContentException e) {
			e.printStackTrace();
		} catch (IllegalLifeCycleException e) {
			e.printStackTrace();
		}
		/*----------------------------------------- DEFINE THE BINDINGS---------------------------*/
		Vector<String> multicastInterfaceNames =getMulticastItfNumber(_owner);

		 // retrieve the set of interfaces belonging to the current worker 
		Vector<MulticastBinding> mb=null; // stores the <multicast_ref, serverItf_ref> couples
		try {
			 mb= getPairInterfaces(((MulticastController)_owner.getFcInterface("multicast-controller")), multicastInterfaceNames);
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}
		Interface[] itf = (Interface[]) ((ProActiveComponent) worker).getRepresentativeOnThis().getFcInterfaces();
		 int t=0;
		 int pos;
		 BindingController mapBC=null;
		 try {
			mapBC= (BindingController) _owner.getFcInterface(Constants.BINDING_CONTROLLER);
		 } catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}
		 // check for binding compatibility and bind
		 for (t=0;t<mb.size();t++){
			 // search the interface worker_ref in the new worker set of interfaces
			 pos = searchForItf(mb.get(t).worker_ref,itf);
			 String multicast_name = mb.get(t).multicast_ref;
			 if (pos<itf.length)
				 bind(multicast_name, itf[pos], mapBC);
			 else
				throw new AutonomicException();
				 
		 }

		 boolean reconf=false;
		 AutonomicController ac=null;
		 try {
			ac = (AutonomicController) worker.getFcInterface("autonomic-controller");
			reconf = true;
		} catch (NoSuchInterfaceException e) {}
		 if (reconf)
			 // Note: the oldconfig.leght refers to the oldconfiguration INCLUDING the 
			 // manager component!
			 reconfigure(oldconfig, worker,ac);
		 else
			System.out.println("\n[MapAddWorker] Stateless computation running, skip redistribution\n");

			System.out.println("\n========================================================================");
			System.out.println("                        [End NF-MapAddWorker]");
			System.out.println("========================================================================\n");
		 
		// restart the application
		(((gridcomp.map.controller.LifeCycleControllerImpl)lcc)).autonomicStartFc();
		return null;
	}
		 
	

private int searchForItf(Interface interface1, Interface[] itf) {
	int i=0;	
	 for (;i<itf.length;i++){
		 if (itf[i].getFcItfName().equals(interface1.getFcItfName()))
			break;
	 }
	 return i;
	}

private void bind(String multicastname, Interface itf, BindingController mapBC){
 /* binds the interface itf to the multicast interface 
  * with name multicastname through the mapBC binding controller*/
	try {
		 mapBC.bindFc(multicastname, itf);
		 //System.out.println("[MapAddWorker] Binding between "+multicastname+" and "+itf.getFcItfName()+" completed...... ");
		 System.out.println("[MapAddWorker] Binding of "+itf.getFcItfName()+" port completed...... ");
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		} catch (IllegalBindingException e) {
			e.printStackTrace();
		} catch (IllegalLifeCycleException e) {
			e.printStackTrace();
		}
 }
 
 private Component newComponent(String adl, String padPath) throws ProActiveException, ADLException, NoSuchInterfaceException
	    {
	        Factory f=null;
			try {
				f = org.objectweb.proactive.core.component.adl.FactoryFactory.getFactory();
			} catch (ADLException e2) {
				e2.printStackTrace();
			}
			HashMap<String,Object> context = new HashMap<String,Object>(1);
			// just put one time if the worker to add is always the same
			
			//Component newC = newComponent(cmp);
			Component newC=null;
			try {
				//ProActiveDescriptor pad = PADeployment.getProactiveDescriptor("local-deployment.xml");
				ProActiveDescriptor pad = PADeployment.getProactiveDescriptor(padPath);
			    
				int machineId= getVirtualNodeIndex(pad);
			    context.put("deployment-descriptor", pad);
				context.put("vn-id", machineId+"");
				//logger.info("I'm going to create the worker!!");
				newC = (Component) f.newComponent(adl, context);
				
				//logger.info("NEW WORKER INSTANTIATED!!");
			} catch (ADLException e1) {
				e1.printStackTrace();
			}
	        return newC;
	    }
 
 private int getVirtualNodeIndex(ProActiveDescriptor pad) {
	 VirtualNode[] vn = (VirtualNode[]) pad.getVirtualNodes();
	 java.util.Vector<Integer> obj = new Vector<Integer>();
	 int p;
	 
	 StringTokenizer st = null;
	 int id;
	 for (int i=0;i<vn.length;i++){
		 if (vn[i].getName().startsWith("slave-node-")){
				 st = new StringTokenizer(vn[i].getName(),"slave-node-");
				 id = Integer.parseInt(st.nextToken());
				 obj.add(new Integer(id));
		}
	 }
	  p = Math.abs(generator.nextInt()%obj.size());
	//  System.out.println("Assigned Virtual Node Id="+obj.get(p));
	  
	  return ((Integer)obj.get(p)).intValue();
}

private class MulticastBinding{
	 String multicast_ref;
	 Interface worker_ref;
	 public MulticastBinding(String s, Interface t){
		 multicast_ref=s;
		 worker_ref = t;
	 }
 }

private void reconfigure(Component[] oldconfig, Component newworker, AutonomicController newac) {
	 // Note: the oldconfig.leght refers to the oldconfiguration INCLUDING the 
	 // manager component!

	//AutonomicController ac=null;
	Object[] o = new Object[1];
	o[0]=new Integer(oldconfig.length+1);
	GenericTypeWrapper t = null;
	List<Object> collect = new ArrayList<Object>();
	// retrieve the current state of the workers
	System.out.println("\n============================================");
	System.out.println("   Redistribution of dataset: "+(oldconfig.length-1)+" -> "+oldconfig.length+" workers ");
	System.out.println("============================================\n");
	
	/*
	 *  COLLECT
	 */ 
	//collects also the workers' controllers, excluding the manager component
	
	System.out.println("\n[MapAddWorker] COLLECT partitions from "+(oldconfig.length-1)+" workers ======\n");
	//try {Thread.sleep(3000);} catch (InterruptedException e1) {e1.printStackTrace();}
	AutonomicController[] acAll = new AutonomicController[oldconfig.length];
	
	int p=0;
	for (int j=0;j<oldconfig.length;j++){
		try {
			acAll[p] = (AutonomicController) oldconfig[j].getFcInterface("autonomic-controller");
			// get the worker status as a List 
			t = (GenericTypeWrapper)acAll[p].execOperation("GetWorkerStatus",o);
			List partition = (List) t.getObject();
			if (partition!=null){
				// deep copy of the status and collapsing
				for (int i=0;i<partition.size();i++)
					collect.add((Task) partition.get(i));
			}	
			p++;
		} catch (NoSuchInterfaceException e) {
			// I've got the manager
		}
	}
	// the new worker will be placed in the last position of acAll
	acAll[p] =  newac;
	
	
	if (collect.size()==0){
		System.out.println("\n[MapAddWorker] ======= No state to redistribute.... Redistribution COMPLETED =======");
	}else{
		/*
		 * REDISTRIBUTION OF THE STATE
		 * */
		
		Task first = (Task)collect.get(0);
		Task last = (Task) collect.get(collect.size()-1);
		System.out.print("\n[MapAddWorker] SPLIT dataset ["+first.getObject()+"-"+last.getObject()+"] of "+collect.size()+" tasks into "+oldconfig.length+" partition\n");
		
	
		List<Object> lo = Partition.split(collect, oldconfig.length);
		Partition.showpartitions(lo);

		// distribute the new state to the workers
		System.out.println("\n[MapAddWorker] DISTRIBUTE "+collect.size()+" tasks to "+oldconfig.length+" workers ==\n");
		for (int j=0;j<acAll.length;j++){
			acAll[j].execOperation("SetWorkerStatus", lo.get(j));
		}

		System.out.println("\n============================================");
		System.out.println("         Redistribution COMPLETED             ");
		System.out.println("============================================\n");
		}
	}	 
}
