package gridcomp.map.port;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import org.objectweb.proactive.core.component.exceptions.ParameterDispatchException;
import org.objectweb.proactive.core.component.type.annotations.multicast.ParamDispatch;
import gridcomp.map.Task;

public class MapDispatch implements ParamDispatch, Serializable {
     
	private static final long serialVersionUID = 1L;

	private List<Object> dispatch(List<?> inputParameter, int nbOutputReceivers)
        throws ParameterDispatchException {
		
		List<Object> result = new ArrayList<Object>();
        
        int[] lenArray = new int[nbOutputReceivers]; // array of partitions' lenghts
    	for (int i=0;i<lenArray.length;i++)
    		lenArray[i] = 0;
    	int oddpartition = 	inputParameter.size()%nbOutputReceivers;
        for (int i=0;i<lenArray.length;i++)
        	lenArray[i] = inputParameter.size()/nbOutputReceivers;
        for (int i=0;i<oddpartition;i++)
        	lenArray[i] = lenArray[i]+1;
        	int base = 0;
        	for (int j=0;j<nbOutputReceivers;j++){
        		Task partition;
        		partition= new Task(makePartition(inputParameter,base,lenArray[j]));
            	base = base +lenArray[j];
        		result.add(partition);
        	}
        return result;
    }

	private List<Object> makePartition(List<?> inputParameter,int base, int partlen){
		/*better version
		*/
		List<Object> content=new ArrayList<Object>();
		if (base+partlen <= inputParameter.size())
			for (int i = 0; i < partlen; i++) 
				content.add(inputParameter.get(base+i));
		return content;
	}
	public List<Object> dispatch(Object inputParameter, int nbOutputReceivers)
        throws ParameterDispatchException {
    //	System.out.println("[MapDispatch] Invocata su "+nbOutputReceivers+" ricevitori");
     
		
		
		
    	if (inputParameter instanceof List) {
			//List<Object> lo = Partition.split((List<Object>)inputParameter, nbOutputReceivers);
    		List<Object> lo =dispatch((List) inputParameter, nbOutputReceivers); 
       		return ((List<Object>)lo);
    	}

        // no dispatch in case of non-list parameters
        List<Object> result = new ArrayList<Object>();

        for (int i = 0; i < nbOutputReceivers; i++) {
            result.add(inputParameter);
        }

        return result;
    }

    /*
     * @see org.objectweb.proactive.core.component.type.annotations.collective.ParamDispatch#getDispatchSize(java.util.List, int)
     */
    private int expectedDispatchSize(List<?> inputParameter, int nbOutputReceivers)
        throws ParameterDispatchException {
        int result = 0;
    	
        //  result = inputParameter.size();
        result=nbOutputReceivers;  
        return result;
    }

    /*
     * @see org.objectweb.proactive.core.component.type.annotations.collective.ParamDispatch#getDispatchSize(java.lang.Object, int)
     */
    public int expectedDispatchSize(Object inputParameter, int nbOutputReceivers)
        throws ParameterDispatchException {
        if (inputParameter instanceof List) {
            return expectedDispatchSize((List) inputParameter, nbOutputReceivers);
        }

        return nbOutputReceivers;
    }

    /*
     * @see org.objectweb.proactive.core.component.type.annotations.collective.ParamDispatch#matchesClientSideParameterType(java.lang.reflect.Type)
     */
    public boolean match(Type clientSideInputParameterType, Type serverSideInputParameterType)
        throws ParameterDispatchException {
        boolean result = false;
        boolean clientSideParamTypeIsParameterizedType = (clientSideInputParameterType instanceof ParameterizedType);
        boolean serverSideParamTypeIsParameterizedType = (serverSideInputParameterType instanceof ParameterizedType);
        Class clientSideClass = null;
        Class clientSideElementsType = null;
        Class clientSideRawType = null;

        Class serverSideClass = null;
        Class serverSideElementsType = null;
        Class serverSideRawType = null;

        if (clientSideParamTypeIsParameterizedType) {
            clientSideRawType = (Class) ((ParameterizedType) clientSideInputParameterType).getRawType();
            if (!(((ParameterizedType)clientSideInputParameterType).getActualTypeArguments().length == 1)) {
                throw new ParameterDispatchException("client side input parameter type " +
                                                     clientSideInputParameterType + " can only be parameterized with one type");
            }
            Type cType = ((ParameterizedType) clientSideInputParameterType).getActualTypeArguments()[0];
            if (cType instanceof ParameterizedType) {
            	clientSideElementsType = (Class)((ParameterizedType)cType).getRawType();
            } else {
            	clientSideElementsType = (Class)cType;
            }
        } else {
            if (clientSideInputParameterType instanceof Class) {
                clientSideClass = (Class) clientSideInputParameterType;
            } else {
                throw new ParameterDispatchException("client side input parameter type " +
                    clientSideInputParameterType + " can only be either a parameterized type or a class");
            }
        }

        if (serverSideParamTypeIsParameterizedType) {
            serverSideRawType = ((Class) ((ParameterizedType) serverSideInputParameterType).getRawType());
            if (!(((ParameterizedType)serverSideInputParameterType).getActualTypeArguments().length == 1)) {
                throw new ParameterDispatchException("server side input parameter type " +
                                                     serverSideInputParameterType + " can only be parameterized with one type");
            }
            Type sType = ((ParameterizedType) serverSideInputParameterType).getActualTypeArguments()[0];
            if (sType instanceof ParameterizedType) {
            	serverSideElementsType = (Class)((ParameterizedType)sType).getRawType();
            } else {
            	serverSideElementsType = (Class)sType;
            }
            
            serverSideElementsType = ((Class) ((ParameterizedType) serverSideInputParameterType).getOwnerType());
        } else {
            if (serverSideInputParameterType instanceof Class) {
                serverSideClass = (Class) serverSideInputParameterType;
            } else {
                throw new ParameterDispatchException("server side input parameter type " +
                    serverSideInputParameterType + " is incompatible with " +
                    "client side input parameter type " + clientSideInputParameterType);
            }
        }
            if (clientSideParamTypeIsParameterizedType) {
                if (clientSideElementsType.isAssignableFrom(serverSideClass)) {
                    result = true;
                } else if (serverSideParamTypeIsParameterizedType) {
                    result = clientSideElementsType.isAssignableFrom(serverSideRawType);
                }
            } else {
                result = false; // maybe this constraint should be softened
            }
       
        return result;
    }
}
