package gridcomp.example.active.dicom.gui;

import java.awt.AWTEventMulticaster;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.image.AreaAveragingScaleFilter;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ImageProducer;
import java.awt.image.IndexColorModel;
import java.awt.image.MemoryImageSource;
import java.net.URL;

import javax.swing.JComponent;

public class ThumbImage extends JComponent {

	/**
	 * 
	 */
	private boolean bGrayComp=false;
	private int gCenter = 1;
	private String label = null;
	private static final long serialVersionUID = 6819224179546232003L;
	protected ActionListener actionListener = null;
	private int w,h;
	private boolean clicked;
	private BufferedImage thumb;
	private Image fullImage;
    private int scaleWidth = 100;
    private int scaleHeight = 80;
	public ThumbImage(Image image) {
		clicked=false;
		InitImage(image);
	//	setSize(w,h);
		this.setPreferredSize(new Dimension(100,130));
		addMouseListener(new ThumbImageButtonMouseListener(image));
	
	}
	
	private BufferedImage convert(Image image) {
		    BufferedImage bi = new BufferedImage(image.getWidth(null),
		                                         image.getHeight(null),
		                                         BufferedImage.TYPE_INT_RGB);
		    Graphics g = bi.getGraphics();
		    g.drawImage(image, 0, 0, null);
		    g.dispose();
		    return bi;
		  }
	public void setScaleWidth(int sw)
	{
		scaleWidth = sw;
	}
	public int getScaleWidth()
	{
		return scaleWidth;
	}
	public void setScaleHeight(int sw)
	{
		scaleHeight = sw;
	}
	public int getScaleHeight()
	{
		return scaleHeight;
	}
	private BufferedImage scaleTo(int width, int height)
	{
		BufferedImage original = convert(fullImage);
		int originalWidth = original.getWidth();
		int originalHeight = original.getHeight();
		float factorX = (float)originalWidth / width;
		float factorY = (float)originalHeight / height;
		factorX = Math.max(factorX, factorY);
		factorY = factorX;
		    
		    // The scaling will be nice smooth with this filter
		    AreaAveragingScaleFilter scaleFilter =
		      new AreaAveragingScaleFilter(Math.round(originalWidth / factorX),
		                                   Math.round(originalHeight / factorY));
		    ImageProducer producer = new FilteredImageSource(original.getSource(),
		                                                     scaleFilter);
		    ImageGenerator generator = new ImageGenerator();
		    producer.startProduction(generator);
		    BufferedImage scaled = generator.getImage();
		    return scaled;
	}
	public void InitImage(Image image) {
		MediaTracker tracker;
		try {
			fullImage = image;
			thumb = scaleTo(100, 80);
			tracker = new MediaTracker(this);
			tracker.addImage(thumb,0);
			tracker.waitForAll();
		} 
		catch (InterruptedException e) {
			e.printStackTrace();
		}
		// Graphics2D devicecontext = fullImage.
		w=thumb.getWidth(this);
		h=thumb.getHeight(this);
		setSize(w,h);
	}

	public void paintComponent(Graphics g) {
		if (thumb!=null)
		{
			g.drawImage(thumb,0,0,this);
			if (label!=null)
			{
			FontMetrics metrics = getFontMetrics(g.getFont());
			int messageWidth = metrics.stringWidth(label);
			int startX = (messageWidth > thumb.getWidth()) ? (messageWidth - thumb.getWidth())/2 : (thumb.getWidth() - messageWidth)/2; 
			
			g.drawString(label,startX, thumb.getHeight()+10);
			}
			
		
		}
	}

	public void addActionListener(ActionListener l) {
		actionListener = 
			AWTEventMulticaster.add(actionListener,l);
	}
	public void removeActionListener(ActionListener l) {
		actionListener =
			AWTEventMulticaster.remove(actionListener, l);
	}

	public void setLabel(String name) {
		label = name;
	}
   public void setPreferredSize(Dimension d)
   {
	   super.setPreferredSize(d);
   }
   public Dimension getPreferredSize()
   {
	   return super.getPreferredSize();
   }

}
