package gridcomp.example.dicom.gui;

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.ColorModel;
import java.awt.image.IndexColorModel;
import java.awt.image.MemoryImageSource;
import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Vector;

public class ImageModelImpl implements ImageModel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1739202747062499503L;
	private Vector<ImageModelListener> listener = new Vector<ImageModelListener>();
	private HashMap<String, Image> map = new HashMap<String, Image>();
	private final int IMAGE_DEFAULT_WIDTH = 256;
	private final int IMAGE_DEFAULT_HEIGHT = 256;
	private final int IMAGE_DEFAULT_SIZE = IMAGE_DEFAULT_WIDTH*IMAGE_DEFAULT_HEIGHT; 
	private boolean bGrayComp = false;
	public void addImage(String name, byte[] completeImage) {
		Image img = decodeDICOMImage(completeImage);
		map.put(name, img);
		for (ImageModelListener l: listener)
		{
			l.update(name);
		}
	}

	public void addModelListener(ImageModelListener l) {
		listener.add(l);
	}

	public void clear() {
		Collection<Image> images = map.values();
		map.clear();
		for (Image i: images)
				i = null;
		}

	public void removeImage(String name) {
		if (map.containsKey(name))
		{
		Image m = map.remove(name);
		m = null;
		}
	}

	public void removeModelListener(ImageModelListener l) {
		listener.remove(l);
	}
	   private Image decodeDICOMImage(byte[] pix)
	   {
		 	ColorModel cm = grayColorModel();
		 	Image image = null;
			byte[] destPixels = new byte[IMAGE_DEFAULT_SIZE] ;
			int max=0;
			if (!(pix==null)) {
				if (bGrayComp){
					for (int i=0;i<IMAGE_DEFAULT_SIZE;i++) {
			 			if (pix[i]>max) max=pix[i];
			 		}
					for (int i=0;i<IMAGE_DEFAULT_SIZE;i++) {
						destPixels[i]=(byte)(((float)pix[i]*255)/max);
					}
				}
				else {
					for (int i = 0 ; i < IMAGE_DEFAULT_SIZE ; i++ ){
						int intVal= pix[i];
						if (intVal>255) intVal=255; 
						if (intVal<0) intVal=0;
						destPixels[i]=(byte)intVal;
					}
				}
		 image= Toolkit.getDefaultToolkit().createImage(new MemoryImageSource(IMAGE_DEFAULT_WIDTH, IMAGE_DEFAULT_WIDTH,cm,destPixels, 0, IMAGE_DEFAULT_WIDTH));  
			}
			return image;
	   }
		 			
		private ColorModel grayColorModel(){
			byte[] r = new byte[256] ;
			for (int i = 0; i <256 ; i++ ) r[i] = (byte)(i & 0xff ) ;
			return (new IndexColorModel(8,256,r,r,r));
		}

		public Image getImage(String name) {
			if (!map.containsKey(name))
					return null;
			return map.get(name);
		}

}
