package gridcomp.example.dicom.io;


import java.awt.* ;
import java.util.*;
import java.io.* ;


public  class DicomHeaderReader{

	 byte[] data ;
	 int index  ;
	 private int dataLength;
	 private static  final boolean debug = false ; 
			
	public static final int MAX_HEADER_SIZE = 10000 ;
	private int maxHeaderSize  ;
	public static final String  ImplicitVRLittleEndian = "1.2.840.10008.1.2" 	;
	public static final int _ImplicitVRLittleEndian = 0 ;
	public static final String  ExplicitVRLittleEndian = "1.2.840.10008.1.2.1" 	;
	public static final int _ExplicitVRLittleEndian = 1 ;
	public static final String  ExplicitVRBigEndian = "1.2.840.10008.1.2.2" 	;
	public static final int _ExplicitVRBigEndian = 2 ;
	public static final int _ImplicitVRBigEndian = 3 ;
	public static final String  JPEGCompression = "1.2.840.10008.1.2.4." 	;
	public static final int _JPEGCompression = 10 ;
	public static final int _notUnderstood = 1000 ;
	private  boolean oneSamplePerPixel = true ;
	private  boolean oneFramePerFile   = true ;
	private int errorDetector = -1 ;
	
	private String  VRString    ;
	private String  xSyntaxUID		= "unknown" ;
	private String  imageType		= "unknown" ;
	private String  studyDate		= "unknown" ;	
	private String  modality		= "unknown" ;
	private String  manufacturer	= "unknown" ;
	private String 	institution		= "unknown" ;
	private String 	physician		= "unknown" ;
	private String  patientName		= "unknown" ;
	private String  patientID		= "unknown" ;
	private String  patientBirthdate= "unknown" ;
	private String  sex				= "unknown" ;
	
	private int numberOfFrames		=  1;		
	private int samplesPerPixel 	=  1;	
	private int	h 					= -1; 	
	private int w 					= -1;	
	private int bitsAllocated		= -1;	
	private int bitsStored 			= -1;		
	private int highBit 			= -1; 	
	private int signed 				= -1;		
	private int size 				= -1;		
	private int n 					= -1;	
	private int VR					= -1;	
	
	private boolean bE = false ;
	

 
	public DicomHeaderReader ( byte [] HeaderData ){
	 	this.data = HeaderData ;
	 	dataLength =  data.length ;
	 	index =0;
	 	initHeaderSize() ;
	 	getVR();
	 	getEssentialData() ;
	 	
	}
	protected void initHeaderSize(){
		maxHeaderSize = MAX_HEADER_SIZE * 1 ;
		if(maxHeaderSize > dataLength) maxHeaderSize = dataLength/2 ;
	
	}
		
	protected void getVR(){
		xSyntaxUID		= getaString (0x0002,0x0010, index);
		debug( "\n-------------�\nTransfertsyntaxUID			" + xSyntaxUID ) ;
		
		if(xSyntaxUID != "Unknown" ){
			debug("  xSyntaxUID   Unknown dans boucle if" ) ;
			xSyntaxUID = xSyntaxUID.trim() ;
			if(xSyntaxUID.equals(ImplicitVRLittleEndian ) )		 VR =	_ImplicitVRLittleEndian ;
			else if (xSyntaxUID.equals(ExplicitVRLittleEndian ) )VR =	_ExplicitVRLittleEndian ;	
			else if (xSyntaxUID.equals(ExplicitVRBigEndian ) )	 VR =	_ExplicitVRBigEndian ;	
			else if (xSyntaxUID.startsWith(JPEGCompression ))    VR =	_JPEGCompression ;		
			else VR = _notUnderstood ;
	
			switch (VR){
				case _ImplicitVRLittleEndian 	: VRString = "ImplicitVRLittleEndian	" 	; break;
				case _ExplicitVRLittleEndian 	: VRString = "ExplicitVRLittleEndian	" 	; break;
				case _ExplicitVRBigEndian 		: VRString = "ExplicitVRBigEndian	" 	; break;
				case _JPEGCompression 			: VRString = "JPEGCompression	" 		; break;
				case _notUnderstood 			: VRString = "not understood	" 			; break;
				default : VRString =" Something curious happened !" ;
			}
			debug( " Value representation  :     		" + VRString ) ;
		}
	
		if( VR ==_notUnderstood){
			
			if ( (data[0] != 'D') || (data[1] != 'I' ) || (data[2] != 'C' )|| (data[3] != 'M' )){
				if(data[0] < data[1] ){ 
					VR= _ImplicitVRBigEndian ;
					debug( "VR is implicitVRBigIndian blabla ") ; 
					bE = true ;
				}
			}
		
		}
	
		
	}
	
	
	protected void getEssentialData(){
	
	imageType 		= getaString (0x0008,0x0008, index);
	studyDate		= getaString (0x0008,0x0020 ,index);
	modality		= getaString (0x0008,0x0060 ,index);
	manufacturer	= getaString (0x0008,0x0070	,index);
	institution		= getaString (0x0008,0x0080	,index);
	physician		= getaString (0x0008,0x0090	,index);
	patientName		= getaString (0x0010,0x0010	,index);
	patientID		= getaString (0x0010,0x0020 ,index);
	patientBirthdate= getaString (0x0010,0x0030	,index);
	sex				= getaString (0x0010,0x0040	,index);

	h = getAnInt(0x0028, 0x0010, index ) ;
		if(h ==-1 ){
			 index = 0 ;
			 h =getAnInt(0x0028, 0x0010) ;
			 debug( "Something is going wrong in this file  !!!, h == -1" );
		}
	w = getAnInt(0x0028, 0x0011, index ) ;
	
		if(w ==-1 ){
			 index = 0 ;
			 w = getAnInt(0x0028, 0x0011) ;
			  debug( "Nouvelle valeur de  w avec  0x0028, 0x0011  : " + w);
			 debug( "Something is gooing wrong in this file  !!!, w == -1" );
		}
	
	bitsAllocated		= getAnInt(0x0028, 0x0100, index ) ;
		debug("bitsAllocated ......."+ bitsAllocated );
		if(bitsAllocated ==-1 ){
			 index = 0 ;
			 bitsAllocated = getAnInt(0x0028, 0x0100) ;
			  debug( "Nouvelle valeur de  bitsAllocated: " + bitsAllocated);
		}
	bitsStored 			= getAnInt(0x0028, 0x0101, index ) ;
		debug("bitsStored ......."+ bitsStored );
		if(bitsStored ==-1 ){
			 index = 0 ;
			 bitsStored = getAnInt(0x0028, 0x0101) ;
			  debug( "Nouvelle valeur de  bitsStored : " + bitsStored);
		}
	highBit 			= getAnInt(0x0028, 0x0102, index ) ;
	debug("highBit ......."+ highBit );
		if(highBit ==-1 ){
			 index = 0 ;
			 highBit = getAnInt(0x0028, 0x0102) ;
			  debug( "Nouvelle valeur de  highBit  : " + highBit);
		}
	signed 				= getAnInt(0x0028, 0x0103, index ) ;
	debug("signed ......."+ signed );
		if(signed ==-1 ){
			 index = 0 ;
			 signed = getAnInt(0x0028, 0x0103) ;
			  debug( "Nouvelle valeur de  signed  : " + signed);
		}
	
	
	
	debug( " ImageType ::: 				" + imageType ) ;
	debug( " studyDate ::: 				" + studyDate ) ;
	debug( " modality ::: 				" + modality ) ;
	debug( " manufacturer ::: 			" + manufacturer ) ;
	debug( " Institution ::: 			" + institution ) ;
	debug( " physician  ::: 			" + physician ) ;
	debug( " patientName ::: 			" + patientName ) ;
	debug( " patientID ::: 				" + patientID ) ;
	debug( " patientBirthdate :::			" + patientBirthdate ) ;
	debug( " sex			 :::		" + sex ) ;
	
	debug( " samplesPerPixel ::: 		" + samplesPerPixel ) ;
	debug( " h ::: 					" + h ) ;
	debug( " w :::  				" + w ) ;
	debug( " bitsAllocated ::: 			" + bitsAllocated ) ;
	debug( " bitsStored ::: 			" + bitsStored ) ;
	debug( " highBit ::: 				" + highBit ) ;
	debug( " signed ::: 				" + signed ) ;


	int pos				= lookForMessagePosition( 0x7Fe0, 0x0010, index);
	int tSize =  samplesPerPixel * w*h*bitsAllocated/8 ;
	if( pos != -1 )  size  =  readMessageLength(pos+4) ;
	debug( " size1 :::		 		" + size ) ;
	int HeaderSize =  pos ;
	debug( "\nHeaderSize    : " + HeaderSize ) ;	
	int figuredSize = HeaderSize + 8 + tSize ;
	errorDetector = dataLength - figuredSize ;
	debug( "Byte difference , error detector: dataLength - figuredSize  : "+ errorDetector ) ;
	
	if(errorDetector == 4 ){
		size = readMessageLength(pos+8) ; 
		errorDetector = dataLength - size ;
	}
		
	if( errorDetector != 0){
			
			
		samplesPerPixel 	= getAnInt(0x0028, 0x0002) ;
				if(samplesPerPixel <0 || samplesPerPixel> 3 ){
			 		 samplesPerPixel = 1 ;
				 	debug( " NEW samplesPerPixel ::: 	" + samplesPerPixel ) ;
			 	}
			 	else if (samplesPerPixel == 1) oneSamplePerPixel = true ;
			 	else oneSamplePerPixel = false ;
			
			
			
			try{numberOfFrames = Integer.parseInt(getaString(0x0028,0x0008)) ; }
			catch(NumberFormatException nFE){ numberOfFrames = 1 ;}
			if (numberOfFrames > 1 ) oneFramePerFile = false ;
			
		tSize = numberOfFrames * tSize * samplesPerPixel; 
		figuredSize = HeaderSize + 8 + tSize ;
		errorDetector = dataLength - figuredSize ;
		if (VR == _JPEGCompression) 
			debug( "_JPEGCompression , can't read that file" ) ;
		
		debug( "2 Byte difference , error detector: "	+ errorDetector 
		+"\n Frame per file  " + 	numberOfFrames 
		+"\n samplesPerPixel "  +		samplesPerPixel) ;
		}
			
	}

	private  int lookForMessagePosition(int groupElement, int dataElement ){
		
		int LenMax = data.length -3;
	 	boolean found = false ;
	 	byte testByte1 = (byte) (groupElement & 0xff);
	 	byte testByte2 = (byte) ((groupElement & 0xff00)>>8);
	 	byte testByte3 = (byte) (dataElement & 0xff);
	 	byte testByte4 = (byte) ((dataElement & 0xff00)>> 8);
		if(bE){
			debug( "Strange file going on here ");
			for ( int j =0 ;  j< LenMax || found ; j++){
	 			if( ( data[j] == testByte2 ) && (data[j+1] == testByte1) && (data[j+2]== testByte4) && (data[j+3]== testByte3)){
	 				found = true ;
	 				return j ;
	 			}
		
			}
		}
		else{
			for ( int j =0 ;  j< LenMax || found ; j++){
	 			if( ( data[j] == testByte1 ) && (data[j+1] == testByte2) && (data[j+2]== testByte3) && (data[j+3]== testByte4)){
	 				found = true ;
	 					return j ;
	 			}
			}
		}
		return -1 ;
	}
	
	private  int lookForMessagePosition(int groupElement, int dataElement, int  j ){
		
		int LenMax = data.length -3;
	 	boolean found = false ;
	 	byte testByte1 = (byte) (groupElement & 0xff);
	 	byte testByte2 = (byte) ((groupElement & 0xff00)>>8);
	 	byte testByte3 = (byte) (dataElement & 0xff);
	 	byte testByte4 = (byte) ((dataElement & 0xff00)>> 8);
	 	
		for ( ;  j< LenMax || found ; j++){
	 		if( ( data[j] == testByte1 ) && (data[j+1] == testByte2) && (data[j+2]== testByte3) && (data[j+3]== testByte4)){
	 				found = true ;
	 				return j ;
	 		}
		}
		return -1 ;
	}
	
	 private int readMessageLength(int i){
		int i0 = 	(int) (data[i ] &0xff);
	 	int i1 =	(int) (data[i + 1 ] &0xff);
	 	int i2 =	(int) (data[i + 2 ] &0xff);
	 	int i3 =	(int) (data[i + 3 ] &0xff);
		return i3<<24| i2<<16 |i1<< 8|i0 ;	
	}
	
	 private int readInt32(int i) { 
		return readMessageLength(i);
	}
	
	 private int readInt16( int i ){
	 		
	 		int  i1 = data[i+1]&0xff ;
	 		int  i0 = data[i]&0xff ;	
	 	 int anInt =  i1<<8|i0 ;
	 	 if (anInt < -1){ anInt= (int)(data[i]*256)&0xff + data[i+1]&0xff ;
	 	  		debug("On relit :attention erreur de VR !!!:" + anInt) ;
	 	  		}
		return anInt ;
	}
	
	private void skip (int length){
		index += length;
	}
	
	
	public int  getAnInt(int groupElement, int dataElement){
		int pos = lookForMessagePosition( groupElement, dataElement);
		if(pos < maxHeaderSize && pos != -1 ){
			if (readMessageLength(pos+4) == 2 )
				return readInt16( pos + 8 );
			else if(readMessageLength(pos+4) == 4 )
				return readInt32(pos + 8 );
			else { 
					int h = readInt16( pos + 8 ); 
					if( h < 2500) return  h ;
					else return -1 ;		
			}
			}
		else return -1 ;	
	}
	
private int  getAnInt(int groupElement, int dataElement, int j){
		int pos = lookForMessagePosition( groupElement, dataElement, j);
		if(pos < maxHeaderSize && pos != -1 ){
			index = pos ;
			if (readMessageLength(pos+4) == 2 )
				return readInt16( pos + 8 );
			else if(readMessageLength(pos+4) == 4 )
				return readInt32(pos + 8 );
			else { 
					int h = readInt16( pos + 8 ); 
					if( h < 2500) return  h ;
					else return -1 ;		
			}
			}
		else return -1 ;	
	}
	
	public int  getSize()	{ return dataLength;}
	
	public int getNumberOfFrames(){ return numberOfFrames ;}
	
	public int getSamplesPerPixels() { return samplesPerPixel ;}
	

	public int  getRows()	{return h ;}
	public int  getColumns(){ return w;}
	public int  getBitAllocated(){return bitsAllocated;}
	public int  getBitStored(){return bitsStored;}
	public int  getHighBit(){return	highBit;}
	public int  getSamplesPerPixel(){return	samplesPerPixel;}
	public int  getPixelRepresentation(){return	signed;}
	public String  getPatientName(){ 	return patientName; 		}
	public String  getPatientBirthdate(){ return patientBirthdate; 	}
	public String  getManufacturer(){ 	return manufacturer ;		}
	public String  getPatientID(){ 		return patientID 	; 		}
	public String  getImageType(){ 		return imageType; 			}
	public String  getStudyDate(){ 		return studyDate; 			}
	public String  getModality(){ 		return modality; 			}
	
	public String  getaString(int groupElement, int dataElement){
		int pos = lookForMessagePosition(groupElement,dataElement);
		if(pos < maxHeaderSize  && pos != -1 ){
			int length = readMessageLength(pos+4);
		
			if (length >256) length = readInt16( pos + 6 ); 
			if (length > 64 || length < 0 ) length = 64 ;
			if (length> (dataLength - pos-8)) length = dataLength -pos -9 ;
			char[] patName = new char[length ];
			pos += 8;
			for (int i = 0; i < length ;i++)
					patName[i] = (char)data[pos++];
				return new String(patName );
			}
		else return " Unknown " ;	
	}
	
	private String  getaString(int groupElement, int dataElement, int j ){
		int pos = lookForMessagePosition(groupElement,dataElement,j);
		if(pos < MAX_HEADER_SIZE && pos != -1 ){
			int length = readMessageLength(pos+4);
			if (length >256)length = readInt16( pos + 6 ); ;
			if (length > 64 || length < 0 ) length = 64 ;
			if (length > (dataLength - pos-8)) length = dataLength -pos -9 ;
			index = pos  ;
			pos += 8;
			char[] patName = new char[length ];
			for (int i = 0; i < length ;i++)
					patName[i] = (char)data[pos++];
				return new String(patName );
			}
		else return " Unknown " ;	
	}
	

	public byte[] getPixels(int w,int h) throws DicomIOException{
		
	
		if (VR == _JPEGCompression ) 
			throw new DicomIOException("DICOM JPEG compression not yet supported ") ;	
			
			if ( w ==  -1){
				throw new DicomIOException("Format not recognized: invalid width") ;		
			}
			if( h == -1) throw new DicomIOException("Format not recognized: invalid height") ;
		
		int ba = getBitAllocated();
			if( ba%8 == 0) { ba = ba/8 ;}
				else ba = (ba+8)/8 ;
		int fileLength = w * h * ba ;
		int offset = dataLength - fileLength ; 		
		byte[]  pixData = new byte[ fileLength ];
			java.lang.System.arraycopy (data, offset, pixData,0, fileLength );	
		return pixData ;
	
	}

	private void  debug(String message){
		if (debug )System.out.println(message);	
	}
}