package gridcomp.example.passive.dicom;

import gridcomp.example.dicom.io.DicomDecoder;
import gridcomp.example.dicom.io.DicomIOException;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.atomic.AtomicInteger;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;
import org.objectweb.proactive.core.util.wrapper.StringWrapper;

public class MasterImpl implements Master, MasterAttributes, BindingController 
{
    private Worker clientItf;
    private String imagesDir;
    private final byte endByte = 0;
    long start, end;
    private int items;
    public void run(int niter)
    {
    	
     	DicomDecoder DD = new DicomDecoder();
  		File dir = new File(imagesDir);
  		File[] files = dir.listFiles();
  		int h = 0;
  		int w = 0;
  		int counter = 0;
  		start=System.currentTimeMillis();
  		
  		if (files == null)
  		{
  		System.err.println("Path directory error. Please check the image path in the adl.");	
  		return;
  		}
  		// if the number of iteration = -1 then number of iteration is the number of available files.
  		if (niter==-1)
  		{
  			niter = files.length;
  		}
  		niter = (niter > files.length) ? files.length : niter;
  		
  		for(int f=0; f<niter; f++) {
		    /* a file is  readable when: exists, i can read it, it's
		       not a dir 
		    */
  			boolean readAble = (files[f]!=null) && files[f].exists() && files[f].canRead() && 
  			!(files[f].getName().startsWith(".") || files[f].getName().startsWith(".."));
  			
  			if (readAble)
  			{
  			try {
  				if (DD!=null)
  					DD.getImageToDisplay(files[f].getAbsolutePath());
  			} catch (DicomIOException e) {
  				// TODO Better exception handling
  				e.printStackTrace();
  				return;
  			}
  			
  				
  		    w = DD.getWid();
  		    h = DD.getHe();

  		    short [] pixels = new short[w*h];

  		    byte [] b = new byte[w*h];
  		    pixels = DD.getIntValue();    // lettura non normalizzata
  		    int max = 0;
  		    for (int i=0;i<(w*h);i++)     // normalizzazione
  			if (pixels[i]>max) 
  			    max=pixels[i];
  		    for (int i=0;i<(w*h);i++) 
  			b[i]=(byte)((pixels[i]*255)/max);
  		    clientItf.computeTask(b,counter);
  		    ++counter;
  		}
  		}
  		byte[] b = new byte[1];
  		b[0] = this.endByte;
  		clientItf.computeTask(b,-1);
      }
    
    public void bindFc(String clientItfName, Object serverItf) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(Worker.ITF_NAME))
                clientItf = (Worker) serverItf;
        else
            throw new NoSuchInterfaceException(clientItfName);
    }

    public String[] listFc() 
    {
        return new String[]{ Worker.ITF_NAME };
    }

    public Object lookupFc(String clientItfName) throws NoSuchInterfaceException 
    {
        if (Worker.ITF_NAME.equals(clientItfName)) 
            return clientItf;
        throw new NoSuchInterfaceException(clientItfName);
    }
    
    public void unbindFc(String clientItfName) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (clientItfName.equals(Worker.ITF_NAME)) 
        	clientItf = null;
        else 
        	throw new NoSuchInterfaceException(clientItfName);
    }
	public String getDirectoryImages() {
		return imagesDir;
	}

	public int getItems() {
		return items;
	}

	public void setItems(int num) {
		items = num;
	}

	public void setDirectoryImages(String dirName) {
		
		imagesDir = dirName; 
	}
}
