package gridcomp.usescase.dicom.sequential;

import java.lang.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;

public class DicomDecoder {
	int wid, he , highBit, n , size ; 
 	boolean signed;
 	final static boolean DEBUG = false ;
 	boolean ignoreNegValues ;
 	int bitsStored, bitsAllocated ;
 	int samplesPerPixel ;
 	int numberOfFrames ;
 	DicomHeaderReader dHR ;
 
 	byte [] pixData;
 	short pixels[];

/* This function load an image to display given a filename
 * 	@param String nomefile
 *  return true or false.
 * */	
	public boolean getImageToDisplay(String nomefile) {
		FileInputStream in;
		File dicom= new File(nomefile);
		long size = dicom.length();	
		byte[] array 	= new byte[(int)size];
		int bytes_read 	= 0;
		try {
		in = new FileInputStream(nomefile);
		} catch (FileNotFoundException e)
		{
			e.printStackTrace();
			return false;
		}
		try {
		while(bytes_read < size){	
				bytes_read += in.read(array, bytes_read, (int)size - bytes_read);
			}
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
	
	
		
		this.dHR  = new DicomHeaderReader(array);
		he              = dHR.getRows() ;
		wid             = dHR.getColumns() ;
		pixData = new byte[he*wid];
		pixels = new short[he*wid];
		highBit 		= dHR.getHighBit() ;
		bitsStored 		= dHR.getBitStored() ;
		bitsAllocated	= dHR.getBitAllocated();
		n 				= (bitsAllocated/8) 	;
		signed 			= (dHR.getPixelRepresentation() == 1) ;
		size 			=  he * wid;
		try {
			pixData 	=  dHR.getPixels(wid,he);	
		}
		catch (Exception e) {
			e.printStackTrace();
		//	System.err.println("Fail getImage()"+e);
			return false;
		}
		
		getImage() ;
		try {
			in.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return true;
	}


	protected  ColorModel grayColorModel(){
		byte[] r = new byte[256] ;
		for (int i = 0; i <256 ; i++ )
			r[i] = (byte)(i & 0xff ) ;
		return (new IndexColorModel(8,256,r,r,r));
	}



	private void to8PerPix(byte[] pixData){
		if(bitsStored<=8){
			int len = wid * he;
			int value = 0;
			for (int i = 0 ; i< len ; i++ ){
				value =	(int)(pixData[i*2])&0xff ;
				pixels[i]=(byte)value;
			}
			return;	
		}
		int value =0;
		if(highBit >= 8){
			for( int i = 0 ; i< pixels.length ; i++){
				value = ((int)( pixData[(2*i)+1] & 0xff )<<8)|(int)( pixData[(2*i)] & 0xff)  ;
				pixels[i] = (short)value  ;
			}
		}
		else if( highBit <= 7){
			for( int i = 0 ; i< pixels.length ; i++){	
				value = ((int)( pixData[(2*i)] & 0xff )<<8)| (int)( pixData[(2*i)+1] & 0xff)  ;
				pixels[i] = (short)value ;
			}
		}	
	}


	private  void signedTo8PerPix	(byte[] pixData){
		short shValue = 0 ;
		int value = 0 ;
		if ( highBit >= 8 ){
			for (int i = 0 ; i < pixels.length ; i++ ){
				shValue = (short)((( pixData[(2*i)+1] & 0xff )<<8)| ( pixData[(2*i)] & 0xff) ) ;
				value =  shValue ;
				if(value<0) value = 0 ;
				pixels[i] = (short) value  ;
			}
		}
		if ( highBit <= 7 ){
			for (int i = 0 ; i < pixels.length ; i++ ){
				shValue = (short)((( pixData[(2*i)+1] & 0xff )<<8)| ( pixData[(2*i)] & 0xff) ) ;
				value = (int) shValue ;
				if(value<0 && ignoreNegValues ) value = 0 ;
				pixels[i] = (short)value  ;
			}
		}
	}


	public void getImage(){
		if ( !signed) {	 
			to8PerPix(pixData) ;
		}
		else if (signed ){	
			signedTo8PerPix(pixData) ;
		}	
	}

 	public int getWid() {return this.wid;}
 	public int getHe() {return this.he;}
 	public int getn() {return n;}
 	public boolean getSign() {return signed;}
 
 	public short[] getIntValue() {
 		short[]  pixData = new short[wid*he];
 		if (!(pixels==null)) {
			java.lang.System.arraycopy (pixels,0, pixData,0,(wid*he));	
		} else pixData=null; 
		return pixData;
 	}

}