package gridcomp.example.active.mandelbrot;

import java.awt.Color;
import java.io.Serializable;
import java.util.Vector;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

public class LineServerImpl implements LineServer, BindingController, Serializable 
{
    private static final long serialVersionUID = 3640640597502876424L;
 
    private final int width = Input.dimension;
    private final int max = 8192;//1250;
    
    private LineCollector collector = null;
    static int counter = 0;
    public LineServerImpl()
    {
    }
    

    @SuppressWarnings("unchecked")
    public void drawLine(int index) 
    {    
        Color[] line = new Color[width];   
        //line = calcPointColor(index,-1.027664756189, -0.361291856485, 2.201053060E-3);
        line = calcPointColor(index,-0.756423894629558,0.064179410844781,0.000000003300000);
//-0.229337806708, 0.856707007451, 1.215691182E-3);//-0.562248070717, -0.642846029168, 9.56426089E-4);//-0.565288411853, -0.6514590056883, 3.7234507847E-2);//-0.7722767459658476, 0.1240001223589245, 7.450580596923828E-2);
        Vector res = new Vector();
        res.add(line);
        res.add(index);        
        collector.commitLine(line, index);
        // System.out.println("Committed line " + (++counter));
    }
      
    private Color[] calcPointColor(int index,double cx, double cy, double size) 
    {
         double xc   = cx;
         double yc   = cy;

         Color[] line = new Color[width];
        
         for (int j = 0; j < width; j++) 
         {
             double x0 = xc - size/2 + size*index/width;
             double y0 = yc + size/2 - size*j/width;
             Complex z0 = new Complex(x0, y0);
             int gray = max - mand(z0, max);
             Color color = null;
        
             if (gray == 0) 
                 gray = max;     
             //if (gray < 1100)   
             //   color = Color.getHSBColor((gray / 1250.0f), 0.8f, 1.0f);
             //else
             //   color = Color.getHSBColor((gray / 1250.0f), 0.8f, ((1250 - gray)/150.0f));
             color = Color.getHSBColor( ((float) gray%64)/64, ((float) gray%128)/128, ((float) gray%254)/254); 
             line[j] = color;
         }
         return line;
    }
        
    int mand(Complex z0, int max) 
    {
        Complex z = z0;
        for (int t = 0; t < max; t++) { 
            if (z.abs() > 2) 
                return t;
            z = z.times(z).plus(z0);
        }
        return max;
    }
     
    
    public void bindFc(String arg0, Object arg1) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if (arg0.equals("collector-client"))
            collector = (LineCollector) arg1;
        else throw new NoSuchInterfaceException("No such interface named: "+arg0);
    }

    public String[] listFc() 
    {
        return new String[]{"collector-client"};
    }

    public Object lookupFc(String arg0) throws NoSuchInterfaceException 
    {
        if (arg0.equals("collector-client"))
            return collector;
        throw new NoSuchInterfaceException("No such interface named: "+arg0);
    }

    public void unbindFc(String arg0) throws NoSuchInterfaceException, IllegalBindingException, IllegalLifeCycleException 
    {
        if(arg0.equals("collector-client"))
            collector = null;
        else throw new NoSuchInterfaceException("No such interface named: "+arg0);
    }
}
