package gridcomp.controller;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import gridcomp.AutonomicException;
import gridcomp.Constants;
import gridcomp.farm.port.CollectiveInterfaceDispatcherProxy;
import gridcomp.log.Loggers;
import gridcomp.operation.FarmAddWorker;
import gridcomp.operation.FarmNumWorkers;
import gridcomp.operation.FarmQueueWorkers;


import org.apache.log4j.Logger;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.proactive.core.component.Fractive;
import org.objectweb.proactive.core.component.ProActiveInterface;
import org.objectweb.proactive.core.component.identity.ProActiveComponent;
import org.objectweb.proactive.core.component.type.ProActiveInterfaceType;

public class FarmController extends AbstractAutonomicController
{    
	private static final long serialVersionUID = 1L;
	private static final Logger logger = Logger.getLogger(Loggers.CONTROLLER);
		    
    public FarmController(Component owner) throws AutonomicException
    {
        super(owner);
        addAutonomicOperation(FarmAddWorker.OPNAME, new FarmAddWorker(this));
        addAutonomicOperation(FarmNumWorkers.OPNAME, new FarmNumWorkers(this));
        addAutonomicOperation(FarmQueueWorkers.OPNAME, new FarmQueueWorkers(this));
    }
    
    /*
    public void init()
    {
        try {
        	this.getKnowledge().put(KnowledgeConstants.PARENT, owner);
            // Here we check the controlled component contains only a single, primitive, "worker" component
            try {
                Component[] list = Fractal.getContentController(owner).getFcSubComponents();
                if (list == null)
                    throw new AutonomicException("Composite component with no subcomponents?!?!?!");
                if (list.length != 1)
                    throw new AutonomicException("Composite component with more than 1 subcomponent!");
                this.getKnowledge().put(KnowledgeConstants.NUM_WORKERS, list.length);
            } catch (AutonomicException ae) {
                throw ae;
            }   
        } catch (NoSuchInterfaceException e) {
            e.printStackTrace();
            throw new AutonomicException("Exception in initializeController() method.", e);
        }    	
    }
    */
    
    @Override
    public void autonomicInit(File beske) throws Exception
    {
    	Map<String, CollectiveInterfaceDispatcherProxy> itfNameToCollectiveItfProxy = new HashMap<String, CollectiveInterfaceDispatcherProxy>();
    	int numWorkers = 0;
    	
    	// Retrieve the life cycle controller of the component
    	ProActiveComponent owner = (ProActiveComponent) (this.knowledgeMap.get(KnowledgeConstants.PARENT));
        Object[] ownerItfs = owner.getRepresentativeOnThis().getFcInterfaces();
        for (Object otf : ownerItfs) 
        {
            Interface itf = (Interface)otf;
            InterfaceType itfType = (InterfaceType) itf.getFcItfType();
            String        itfName = itf.getFcItfName(); 
            if (!itfType.isFcClientItf() && !itfName.endsWith("-controller") && !itfName.equals("component")) 
            {
                // 1. Collective interface
                ProActiveInterface collItf = null;
                try {
                    collItf = Fractive.createCollectiveClientInterface(itfName, itfType.getFcItfSignature(), ((ProActiveComponent)owner).getRepresentativeOnThis());
                    logger.info("FarmAddWorker:: Created internal collective client interface for external server interface [" + itfName + "].");
                } catch (Exception e) { e.printStackTrace(); }

                // 2. Proxy
                CollectiveInterfaceDispatcherProxy collPrx = null;
                try {
                    collPrx = new CollectiveInterfaceDispatcherProxy();
                    collPrx.setInterfaceType( (ProActiveInterfaceType) itfType );
                    logger.info("FarmAddWorker:: Created collective interface dispatcher proxy for external server interface [" + itfName + "].");
                } catch (Exception e) { e.printStackTrace(); }
                itfNameToCollectiveItfProxy.put(itf.getFcItfName(), collPrx);
                
                // Adding to the proxy members the workers present
                ContentController cc = (ContentController) owner.getFcInterface(Constants.CONTENT_CONTROLLER);
                Component[] comps = cc.getFcSubComponents();
                numWorkers = comps.length;
                for (Component c: comps)
                	collPrx.add(c);
                // Setting the proxy for the collective interface we just created
                collItf.setProxy(collPrx);
                
                BindingController bc = (BindingController) owner.getFcInterface(Constants.BINDING_CONTROLLER);
                // We unbind the old client interface...
                logger.debug("Questo interfaccia-->"+itf.getFcItfName());
                bc.unbindFc(itf.getFcItfName());
                // ... and we bind the new one
                bc.bindFc(itf.getFcItfName(), collItf);
            }            
        }
        // We make the proxy table available to the controller (because of the FarmSubWorker)
        this.knowledgeMap.put(KnowledgeConstants.PROXY_MAP, itfNameToCollectiveItfProxy);
        this.knowledgeMap.put(KnowledgeConstants.NUM_WORKERS, numWorkers);
        logger.info("FarmController::Collective Interfaces successfully initialized.");
    }
}
