package gridcomp.farm.gui;

import gridcomp.Constants;
import gridcomp.controller.AutonomicController;
import gridcomp.controller.BeskeAttributeController;
import gridcomp.operation.FarmAddWorker;
import gridcomp.operation.FarmBalanceWorkers;
import gridcomp.operation.FarmMonitor;
import gridcomp.operation.FarmSubWorker;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Method;
import java.text.DecimalFormat;

import javax.swing.*;
import javax.swing.border.*;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.proactive.core.component.controller.QueueController;
import org.objectweb.proactive.core.util.wrapper.DoubleWrapper;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;
import org.jdesktop.swingworker.SwingWorker;


public class JFarm extends JPanel // implements ActionListener
{
	private static final long serialVersionUID = 1L;
		
	private final int NUM_WORKERS = 3;
	private static final String NAME_PREFIX = "slave-";
	private static DecimalFormat df = new DecimalFormat("0.00");
    
	private   Component                farm = null;
    private   AutonomicController      abc  = null;
    protected BeskeAttributeController bac  = null;

    private int index = 0;

    private JPanel workersPanel;
    
    private JPanel  wPanel[]  = new JPanel[NUM_WORKERS];
    private JButton wButton[] = new JButton[NUM_WORKERS];
    private JLabel  qLabel[]  = new JLabel[NUM_WORKERS];
    
    private JPanel controlPanel1, controlPanel2, spacer1, spacer2;
    private JButton addButton, getButton, removeButton, balanceButton;
    
    private JPanel monitorPanel;
    private JPanel vSpacer1, vSpacer2, vSpacer3, vSpacer4, vSpacer5, vSpacer6;
    private JPanel serverLabelPanel, clientLabelPanel;
    private JLabel serverItfLabel, serverArrLabel, serverDepLabel, clientItfLabel, clientSendLabel, clientRecvLabel;
    private JPanel serverDataPanel, clientDataPanel; 
    private JLabel serverItfData, serverArrData, serverDepData, clientItfData, clientSendData, clientRecvData;
    private JPanel monitorControlPanel;
    private JButton monitorButton;

    
    public JFarm(Component owner, String name, boolean monitor)
    {
    	farm = owner;
        try {
			abc = (AutonomicController) farm.getFcInterface(Constants.AUTONOMIC_CONTROLLER);
	        bac = (BeskeAttributeController) farm.getFcInterface(Constants.BESKE_ATTRIBUTE_CONTROLLER);
		} catch (NoSuchInterfaceException e) {
			e.printStackTrace();
		}
    	    	
        initComponents(monitor);
        initButtonListeners();
        if (monitor)
        	initMonitoringData();
    }

    private void initComponents(boolean monitor) 
    {
        workersPanel = new JPanel();
        for (int i = 0; i < NUM_WORKERS; i++) {
            wPanel[i]  = new JPanel();
            wButton[i] = new JButton();
            qLabel[i]  = new JLabel();
        }
        controlPanel1 = new JPanel();
        controlPanel2 = new JPanel();
        addButton = new JButton();
        getButton = new JButton();
        removeButton = new JButton();
        balanceButton = new JButton();
        spacer1 = new JPanel();
        spacer2 = new JPanel();
        
        monitorPanel = new JPanel();
        vSpacer1 = new JPanel(null);
        vSpacer2 = new JPanel(null);
        serverLabelPanel = new JPanel();
        serverItfLabel = new JLabel();
        serverArrLabel = new JLabel();
        serverDepLabel = new JLabel();
        serverDataPanel = new JPanel();
        serverItfData = new JLabel();
        serverArrData = new JLabel();
        serverDepData = new JLabel();
        vSpacer3 = new JPanel(null);
        clientLabelPanel = new JPanel();
        clientItfLabel = new JLabel();
        clientSendLabel = new JLabel();
        clientRecvLabel = new JLabel();
        clientDataPanel = new JPanel();
        clientItfData = new JLabel();
        clientSendData = new JLabel();
        clientRecvData = new JLabel();
        vSpacer4 = new JPanel(null);
        monitorControlPanel = new JPanel();
        monitorButton = new JButton();
        vSpacer5 = new JPanel(null);
        vSpacer6 = new JPanel(null);

        
        setLayout(new GridBagLayout());
        ((GridBagLayout)getLayout()).columnWidths = new int[] {225, 0, 0};
        ((GridBagLayout)getLayout()).rowHeights   = new int[] {0, 0, 0, 0, 0, 0};
        ((GridBagLayout)getLayout()).columnWeights = new double[] {0.0, 0.0, 0.0};
        ((GridBagLayout)getLayout()).rowWeights    = new double[] {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};

        //======== workersPanel ========
        {
            workersPanel.setBorder(new TitledBorder(null, "Farm Controller", TitledBorder.TRAILING, TitledBorder.BOTTOM));
            workersPanel.setLayout(new BoxLayout(workersPanel, BoxLayout.Y_AXIS));

            for (int i = 0; i < NUM_WORKERS; i++) {
                //======== wPanel ========
                {
                    wPanel[i].setLayout(new BoxLayout(wPanel[i], BoxLayout.X_AXIS));
                    
                    //---- wButton ----
                    wButton[i].setText("W" + (i+1));
                    wButton[i].setMaximumSize(new Dimension(100, 30));
                    wButton[i].setMinimumSize(new Dimension(100, 30));
                    wButton[i].setPreferredSize(new Dimension(100, 30));
                    wButton[i].setEnabled(false);
                    wButton[i].setFocusable(false);
                    wPanel[i].add(wButton[i]);
                    
                    //---- qLabel ----
                    qLabel[i].setText("");
                    qLabel[i].setBorder(new EtchedBorder());
                    qLabel[i].setMinimumSize(new Dimension(100, 30));
                    qLabel[i].setMaximumSize(new Dimension(100, 30));
                    qLabel[i].setPreferredSize(new Dimension(100, 30));
                    qLabel[i].setHorizontalAlignment(SwingConstants.RIGHT);
                    wPanel[i].add(qLabel[i]);
                }
                wButton[0].setEnabled(true);
                workersPanel.add(wPanel[i]);
            }
            //======== controlPanel1 ========
            {
                controlPanel1.setLayout(new BoxLayout(controlPanel1, BoxLayout.X_AXIS));

                //---- addButton ----
                addButton.setText("add");
                // addButton.addActionListener(this);
                controlPanel1.add(addButton);
                controlPanel1.add(spacer1);

                //---- getButton ----
                getButton.setText("get");
                // getButton.addActionListener(this);
                controlPanel1.add(getButton);
            }
            workersPanel.add(controlPanel1);

            //======== controPanel2 ========
            {
                controlPanel2.setLayout(new BoxLayout(controlPanel2, BoxLayout.X_AXIS));

                //---- removeButton ----
                removeButton.setText("remove");
                removeButton.setEnabled(false);
                // removeButton.addActionListener(this);
                controlPanel2.add(removeButton);
                controlPanel2.add(spacer2);

                // check if balancing or not
                String ops[] = abc.listAutonomicOperations();
                abc.execOperation(FarmBalanceWorkers.OPNAME);
                for (String op: ops) 
                    if (op.equals(FarmBalanceWorkers.OPNAME)) {
                        //---- balanceButton ----
                        balanceButton.setText("balance");
                        // balanceButton.addActionListener(this);
                        controlPanel2.add(balanceButton);
                        break;
                    }
            }
            workersPanel.add(controlPanel2);
        }
        add(workersPanel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 5), 0, 0));
        
        if (monitor) {
        //======== monitorPanel ========
        {
            monitorPanel.setBorder(new TitledBorder(null, "Farm Monitor", TitledBorder.TRAILING, TitledBorder.BOTTOM));
            monitorPanel.setLayout(new BoxLayout(monitorPanel, BoxLayout.Y_AXIS));
            monitorPanel.add(vSpacer1);
            monitorPanel.add(vSpacer2);

            //======== serverLabelPanel ========
            {
                serverLabelPanel.setLayout(new BoxLayout(serverLabelPanel, BoxLayout.X_AXIS));

                //---- serverItfLabel ----
                serverItfLabel.setText("Server Interface");
                serverItfLabel.setBorder(null);
                serverItfLabel.setMaximumSize(new Dimension(120, 30));
                serverItfLabel.setMinimumSize(new Dimension(120, 30));
                serverItfLabel.setPreferredSize(new Dimension(120, 20));
                serverItfLabel.setHorizontalAlignment(SwingConstants.CENTER);
                serverLabelPanel.add(serverItfLabel);

                //---- serverArrLabel ----
                serverArrLabel.setText("Arr. rate");
                serverArrLabel.setBorder(null);
                serverArrLabel.setMinimumSize(new Dimension(70, 30));
                serverArrLabel.setMaximumSize(new Dimension(70, 30));
                serverArrLabel.setPreferredSize(new Dimension(70, 30));
                serverArrLabel.setHorizontalAlignment(SwingConstants.CENTER);
                serverLabelPanel.add(serverArrLabel);

                //---- serverDepLabel ----
                serverDepLabel.setText("Dep. rate");
                serverDepLabel.setBorder(null);
                serverDepLabel.setMinimumSize(new Dimension(60, 30));
                serverDepLabel.setMaximumSize(new Dimension(60, 30));
                serverDepLabel.setPreferredSize(new Dimension(60, 30));
                serverDepLabel.setHorizontalAlignment(SwingConstants.CENTER);
                serverLabelPanel.add(serverDepLabel);
            }
            monitorPanel.add(serverLabelPanel);

            //======== serverDataPanel ========
            {
                serverDataPanel.setLayout(new BoxLayout(serverDataPanel, BoxLayout.X_AXIS));

                //---- serverItfData ----
                serverItfData.setText("");
                serverItfData.setBorder(new BevelBorder(BevelBorder.RAISED));
                serverItfData.setMaximumSize(new Dimension(120, 30));
                serverItfData.setMinimumSize(new Dimension(120, 30));
                serverItfData.setPreferredSize(new Dimension(120, 20));
                serverItfData.setHorizontalAlignment(SwingConstants.CENTER);
                serverDataPanel.add(serverItfData);

                //---- serverArrData ----
                serverArrData.setText("");
                serverArrData.setBorder(new BevelBorder(BevelBorder.LOWERED));
                serverArrData.setMinimumSize(new Dimension(70, 30));
                serverArrData.setMaximumSize(new Dimension(70, 30));
                serverArrData.setPreferredSize(new Dimension(70, 30));
                serverArrData.setHorizontalAlignment(SwingConstants.CENTER);
                serverDataPanel.add(serverArrData);

                //---- serverDepData ----
                serverDepData.setText("");
                serverDepData.setBorder(new BevelBorder(BevelBorder.LOWERED));
                serverDepData.setMinimumSize(new Dimension(60, 30));
                serverDepData.setMaximumSize(new Dimension(60, 30));
                serverDepData.setPreferredSize(new Dimension(60, 30));
                serverDepData.setHorizontalAlignment(SwingConstants.CENTER);
                serverDataPanel.add(serverDepData);
            }
            monitorPanel.add(serverDataPanel);
            monitorPanel.add(vSpacer3);

            //======== clientLabelPanel ========
            {
                clientLabelPanel.setLayout(new BoxLayout(clientLabelPanel, BoxLayout.X_AXIS));

                //---- clientItfLabel ----
                clientItfLabel.setText("Client Interface");
                clientItfLabel.setBorder(null);
                clientItfLabel.setMaximumSize(new Dimension(120, 30));
                clientItfLabel.setMinimumSize(new Dimension(120, 30));
                clientItfLabel.setPreferredSize(new Dimension(120, 20));
                clientItfLabel.setHorizontalAlignment(SwingConstants.CENTER);
                clientLabelPanel.add(clientItfLabel);

                //---- clientSendLabel ----
                clientSendLabel.setText("Send rate");
                clientSendLabel.setBorder(null);
                clientSendLabel.setMinimumSize(new Dimension(70, 30));
                clientSendLabel.setMaximumSize(new Dimension(70, 30));
                clientSendLabel.setPreferredSize(new Dimension(70, 30));
                clientSendLabel.setHorizontalAlignment(SwingConstants.CENTER);
                clientLabelPanel.add(clientSendLabel);

                //---- clientRecvLabel ----
                clientRecvLabel.setText("Rec. rate");
                clientRecvLabel.setBorder(null);
                clientRecvLabel.setMinimumSize(new Dimension(60, 30));
                clientRecvLabel.setMaximumSize(new Dimension(60, 30));
                clientRecvLabel.setPreferredSize(new Dimension(60, 30));
                clientRecvLabel.setHorizontalAlignment(SwingConstants.CENTER);
                clientLabelPanel.add(clientRecvLabel);
            }
            monitorPanel.add(clientLabelPanel);

            //======== clientDataPanel ========
            {
                clientDataPanel.setLayout(new BoxLayout(clientDataPanel, BoxLayout.X_AXIS));

                //---- clientItfData ----
                clientItfData.setText("");
                clientItfData.setBorder(new BevelBorder(BevelBorder.RAISED));
                clientItfData.setMaximumSize(new Dimension(120, 30));
                clientItfData.setMinimumSize(new Dimension(120, 30));
                clientItfData.setPreferredSize(new Dimension(120, 20));
                clientItfData.setHorizontalAlignment(SwingConstants.CENTER);
                clientDataPanel.add(clientItfData);

                //---- clientSendData ----
                clientSendData.setText("");
                clientSendData.setBorder(new BevelBorder(BevelBorder.LOWERED));
                clientSendData.setMinimumSize(new Dimension(70, 30));
                clientSendData.setMaximumSize(new Dimension(70, 30));
                clientSendData.setPreferredSize(new Dimension(70, 30));
                clientSendData.setHorizontalAlignment(SwingConstants.CENTER);
                clientDataPanel.add(clientSendData);

                //---- clientRecvData ----
                clientRecvData.setText("");
                clientRecvData.setBorder(new BevelBorder(BevelBorder.LOWERED));
                clientRecvData.setMinimumSize(new Dimension(60, 30));
                clientRecvData.setMaximumSize(new Dimension(60, 30));
                clientRecvData.setPreferredSize(new Dimension(60, 30));
                clientRecvData.setHorizontalAlignment(SwingConstants.CENTER);
                clientDataPanel.add(clientRecvData);
            }
            monitorPanel.add(clientDataPanel);
            monitorPanel.add(vSpacer4);

            //======== monitorControlPanel ========
            {
                monitorControlPanel.setLayout(new BoxLayout(monitorControlPanel, BoxLayout.X_AXIS));

                //---- mGetButton ----
                monitorButton.setText("monitor");
                // monitorButton.addActionListener(this);
                monitorButton.setEnabled(true);
                monitorControlPanel.add(monitorButton);

            }
            monitorPanel.add(monitorControlPanel);
            monitorPanel.add(vSpacer5);
            monitorPanel.add(vSpacer6);
        }
        add(monitorPanel, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
            GridBagConstraints.CENTER, GridBagConstraints.BOTH,
            new Insets(0, 0, 5, 0), 0, 0));
        }
    }
    
    private void initButtonListeners()
    {
    	this.addButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e) 
    		{
    			SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() 
    			{
    				public Void doInBackground() {
    		        	add(++index);
    		        	
    		            wButton[index].setEnabled(true);
    		            qLabel[index].setText("0");
    		            removeButton.setEnabled(true);
    		        	if (index == NUM_WORKERS)
    		                addButton.setEnabled(false);
    		        	refreshControlLabels();
    					return null;
    				}
    				public void done() {
    				}
    			};
    			worker.execute();
    		}
    	});

    	this.removeButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e) 
    		{
    			SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() 
    			{
    				public Void doInBackground() {
    		        	sub(index);

    		            wButton[index].setEnabled(false);
    		            qLabel[index].setText("");
    		            addButton.setEnabled(true);
    		        	if (--index == 0)
    		                removeButton.setEnabled(false);
    		        	refreshControlLabels();
    					return null;
    				}
    				public void done() {
    				}
    			};
    			worker.execute();
    		}
    	});

    	this.balanceButton.addActionListener(new ActionListener() {
    	     public void actionPerformed(ActionEvent e) 
    	     {
    	    	 SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() 
    	    	 {
    	    		 public Void doInBackground() {
    	    			 abc.execOperation(FarmBalanceWorkers.OPNAME);
    	    			 refreshControlLabels();
    	    			 return null;
    	    		 }
    	    		 public void done() {
    	    		 }
    	    	 };
    	    	 worker.execute();
    	     }
    	});
    	
    	this.getButton.addActionListener(new ActionListener() {
   	     	public void actionPerformed(ActionEvent e) 
   	     	{
   	     		SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() 
   	     		{
   	     			public Void doInBackground() {
   	     				refreshControlLabels();
   	     				return null;
   	     			}
   	     			public void done() {
   	     			}
   	     		};
   	     		worker.execute();
   	     	}
    	});
    	
    	this.monitorButton.addActionListener(new ActionListener() {
    		public void actionPerformed(ActionEvent e) 
   	     	{
    			SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() 
   	    	 	{
    				public Void doInBackground() {
    		        	GenericTypeWrapper<Object> obj;
    		        	DoubleWrapper dd;
    		        	obj = abc.execOperation(FarmMonitor.OPNAME, clientItfData.getText(), "client", FarmMonitor.ARRIVAL_RATE);
    		        	dd = (DoubleWrapper)obj.getObject();
    		        	clientRecvData.setText(df.format(dd.doubleValue()));    	
    		        	obj = abc.execOperation(FarmMonitor.OPNAME, clientItfData.getText(), "client", FarmMonitor.DEPARTURE_RATE);
    		        	dd = (DoubleWrapper)obj.getObject();
    		        	clientSendData.setText(df.format(dd.doubleValue()));    	
    		        	obj = abc.execOperation(FarmMonitor.OPNAME, serverItfData.getText(), "server", FarmMonitor.ARRIVAL_RATE);
    		        	dd = (DoubleWrapper)obj.getObject();
    		        	serverArrData.setText(df.format(dd.doubleValue()));    	
    		        	obj = abc.execOperation(FarmMonitor.OPNAME, serverItfData.getText(), "server", FarmMonitor.DEPARTURE_RATE);
    		        	dd = (DoubleWrapper)obj.getObject();
    		        	serverDepData.setText(df.format(dd.doubleValue()));    	
   	    			 	refreshControlLabels();
   	    			 	return null;
   	    		 	}
   	    		 	public void done() {
   	    		 	}
   	    	 	};
   	    	 	worker.execute();
   	     	}
    	});
    	
    }
    
    /*
    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource() == addButton) {

        	add(++index);
        	
            wButton[index].setEnabled(true);
            qLabel[index].setText("0");
            removeButton.setEnabled(true);
        	if (index == NUM_WORKERS)
                addButton.setEnabled(false);
            
        } else if (e.getSource() == removeButton) {
        	
        	sub(index);

            wButton[index].setEnabled(false);
            qLabel[index].setText("");
            addButton.setEnabled(true);
        	if (--index == 0)
                removeButton.setEnabled(false);
            
        } else if (e.getSource() == getButton) {
        	
        } else if (e.getSource() == balanceButton) {
        	
            abc.execOperation(FarmBalanceWorkers.OPNAME);
            
        } else 
        	
        	if (e.getSource() == monitorButton) {
        	
        	GenericTypeWrapper<Object> obj;
        	DoubleWrapper dd;
        	obj = abc.execOperation(FarmMonitor.OPNAME, clientItfData.getText(), "client", FarmMonitor.ARRIVAL_RATE);
        	dd = (DoubleWrapper)obj.getObject();
        	clientRecvData.setText(df.format(dd.doubleValue()));    	
        	obj = abc.execOperation(FarmMonitor.OPNAME, clientItfData.getText(), "client", FarmMonitor.DEPARTURE_RATE);
        	dd = (DoubleWrapper)obj.getObject();
        	clientSendData.setText(df.format(dd.doubleValue()));    	
        	obj = abc.execOperation(FarmMonitor.OPNAME, serverItfData.getText(), "server", FarmMonitor.ARRIVAL_RATE);
        	dd = (DoubleWrapper)obj.getObject();
        	serverArrData.setText(df.format(dd.doubleValue()));    	
        	obj = abc.execOperation(FarmMonitor.OPNAME, serverItfData.getText(), "server", FarmMonitor.DEPARTURE_RATE);
        	dd = (DoubleWrapper)obj.getObject();
        	serverDepData.setText(df.format(dd.doubleValue()));    	
        	
        }
        
        refreshControlLabels();
    }
    */
    
    protected void add(int id)
    {
    	String adl = bac.getProperty("adl.dir").replace('/', '.') + "." + NAME_PREFIX + id;
    	String xml = bac.getProperty("xml.file");
    	abc.execOperation(FarmAddWorker.OPNAME, adl, xml, NAME_PREFIX + id);
    }
    
    protected void sub(int id)
    {
    	abc.execOperation(FarmSubWorker.OPNAME, NAME_PREFIX + id);
    }

    private void refreshControlLabels() 
    {
        Component workers[];
        try {
            workers = Fractal.getContentController(farm).getFcSubComponents();
            for (Component worker: workers) {
                QueueController qc = (QueueController) worker.getFcInterface(Constants.REQUEST_QUEUE_CONTROLLER);
                int num = qc.getSize().intValue();
                String name = Fractal.getNameController(worker).getFcName();
                if (!name.startsWith(NAME_PREFIX)) {
                	qLabel[0].setText("" + num);
                } else {
                	int i = Integer.parseInt(name.split("-")[1]);
                	qLabel[i].setText("" + num);
                }
            }
        } catch (NoSuchInterfaceException e1) {
            e1.printStackTrace();
        }	
    }
    
    private void initMonitoringData() 
    {
    	Object[] objs = farm.getFcInterfaces();
    	for (Object o: objs) {
    		Interface itf = (Interface)o;
    		InterfaceType itfType = (InterfaceType) itf.getFcItfType(); 
    		String itfName = itf.getFcItfName();
            try {
                Class<?> klass = ClassLoader.getSystemClassLoader().loadClass(itfType.getFcItfSignature());
                Method[] methods = klass.getDeclaredMethods();
                if (!itfName.endsWith("-controller") && !itfType.getFcItfName().equals("component"))  
                		if (!itfType.isFcClientItf()) {
                			for (Method met: methods) {
                				serverItfData.setText(met.getName());                			
                			}
                		} else {
                			for (Method met: methods) {
                				clientItfData.setText(met.getName());                			
                			}                	
                		}
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }

    	}
    }
}
