package gridcomp.farm.port;

import gridcomp.log.Loggers;

import java.lang.reflect.InvocationTargetException;

import org.apache.log4j.Logger;
import org.objectweb.proactive.Body;
import org.objectweb.proactive.api.PAActiveObject;
import org.objectweb.proactive.core.component.group.ProxyForComponentInterfaceGroup;

// import org.objectweb.proactive.core.exceptions.manager.NFEManager;
// import org.objectweb.proactive.core.exceptions.proxy.FailedGroupRendezVousException;

import org.objectweb.proactive.core.group.ExceptionListException;
import org.objectweb.proactive.core.group.ProcessForOneWayCall;

import org.objectweb.proactive.core.mop.ConstructionOfReifiedObjectFailedException;
import org.objectweb.proactive.core.mop.MethodCall;

public class CollectiveInterfaceDispatcherProxy extends ProxyForComponentInterfaceGroup 
{
	private static final long serialVersionUID = 1L;

	private int currentWorker = 0; 
    private static final Logger logger = Logger.getLogger(Loggers.COLLECTIVE);
    
    public CollectiveInterfaceDispatcherProxy() throws ConstructionOfReifiedObjectFailedException
    {
        super();
    }
    
    @Override
    protected Object asynchronousCallOnGroup(MethodCall mc) throws InvocationTargetException 
    {        
        Body body = PAActiveObject.getBodyOnThis();
            
        int nextWorker = nextWorker();
        logger.info("New Asynch Call on Group with mc [" + mc.getName() + "] dispatched to worker # " + nextWorker);
        ProcessForSingleAsynchronousCall pfac = new ProcessForSingleAsynchronousCall(this, this.memberList, null, nextWorker, mc, body);
        this.threadpool.addAJob(pfac);
        
        Object ret = pfac.getResult();

        if (ret instanceof Throwable) {
            ((Throwable) ret).printStackTrace();
            throw new InvocationTargetException((Throwable) ret);
        }
    
        return ret;
    }

    @Override
    protected void oneWayCallOnGroup(MethodCall mc, ExceptionListException exceptionList) throws InvocationTargetException 
    {
        Body body = PAActiveObject.getBodyOnThis();
        
        int nextWorker = nextWorker();
        logger.info("New One Way Call on Group with mc [" + mc.getName() + "] dispatched to worker # " + nextWorker);

        this.threadpool.addAJob(new ProcessForOneWayCall(this, this.memberList, nextWorker, mc, body, exceptionList));
        
    }

    
    private int nextWorker() 
    {
        return ((currentWorker++) % this.memberList.size());
    }

    // TODO: Controllare se e' uguale a quella della super-classe oppure no....
    @Override
    public synchronized Object reify(MethodCall mc) throws InvocationTargetException  
    {
        ExceptionListException exceptionList = null;

        /* if the method called is toString, apply it to the proxy, not to the members */
        if ("toString".equals(mc.getName())) {
            return this.toString();
        }

        /* if the method called is hashCode, apply it to the proxy, not to the members */
        if ("hashCode".equals(mc.getName())) {
            return new Integer(this.hashCode());
        }

        if ("equals".equals(mc.getName()) && (mc.getNumberOfParameter() == 1)) {
            return new Boolean(this.equals(mc.getParameter(0)));
        }

        /* result will be a stub on a proxy for group representing the group of results */
        Object result = null;

        /* if OneWay : do not construct result */
        if (mc.isOneWayCall()) {
            exceptionList = new ExceptionListException();
            this.oneWayCallOnGroup(mc, exceptionList);
        }
        /* Special case : the method returns void but is Synchronous because it throws Exception */
        else if (mc.getReifiedMethod().getReturnType() == Void.TYPE) {
            exceptionList = new ExceptionListException();
            this.oneWayCallOnGroup(mc, exceptionList);
        }
        /* if the call is asynchronous the group of result will be a group a future */
        else { // with group in general case : SYNC == ASYNC !!!!
            result = this.asynchronousCallOnGroup(mc);
        }

        /* A barrier of synchronisation to be sur that all calls are done before continuing the execution */
        this.threadpool.complete();

        /* Throws the exceptionList if one or more exceptions occur in the oneWayCall */
        /* Removed because of 3.90 beta
        if ((exceptionList != null) && (exceptionList.size() != 0)) {
            FailedGroupRendezVousException fgrve = new FailedGroupRendezVousException("RendezVous failed in group method: " + mc.getName(), exceptionList, this);
            NFEManager.fireNFE(fgrve, this);
        }
        */
        return result;
    }
    
}
