package gridcomp.farm.port;

import java.io.Serializable;
import java.util.Vector;

import org.objectweb.proactive.Body;
import org.objectweb.proactive.core.body.Context;
import org.objectweb.proactive.core.body.LocalBodyStore;
import org.objectweb.proactive.core.body.proxy.UniversalBodyProxy;
import org.objectweb.proactive.core.component.ProActiveInterface;
import org.objectweb.proactive.core.component.representative.ProActiveComponentRepresentative;
import org.objectweb.proactive.core.group.AbstractProcessForGroup;
import org.objectweb.proactive.core.group.ProcessForAsyncCall;
import org.objectweb.proactive.core.group.ProxyForGroup;
import org.objectweb.proactive.core.mop.MethodCall;
import org.objectweb.proactive.core.mop.Proxy;
import org.objectweb.proactive.core.mop.StubObject;

public class ProcessForSingleAsynchronousCall extends ProcessForAsyncCall implements Serializable
{
    private static final long serialVersionUID = 3712472239136883625L;

    @SuppressWarnings("unchecked")
    private Vector memberListOfResultGroup;
    private int index;
    private MethodCall mc;
    private Body body;
    
    private Object result;
    private Object monitor = new Object();
    private boolean complete = false;
    
    public ProcessForSingleAsynchronousCall(ProxyForGroup proxyGroup, Vector<?> memberList, Vector<?> memberListOfResultGroup, int index, MethodCall mc, Body body) 
    {
        super(proxyGroup, memberList, memberListOfResultGroup, index, mc, body);
        this.proxyGroup = proxyGroup;
        this.memberList = memberList;
        this.memberListOfResultGroup = memberListOfResultGroup;
        this.index = index;
        this.mc = mc;
        this.body = body;
    }
    
    public void run() 
    {    
        Object object = this.memberList.get(this.index);
        LocalBodyStore.getInstance().pushContext(new Context(body, null));;
        boolean objectIsLocal = false;

        /* only do the communication (reify) if the object is not an error nor an exception */
        if (!(object instanceof Throwable)) {
            try {
                if (object instanceof ProActiveComponentRepresentative) {               
                    // delegate to the corresponding interface
                    Object target;
                    if (mc.getComponentMetadata().getComponentInterfaceName() == null) {
                        // a call on the Component interface
                        target = object;
                    } else {
                        target = ((ProActiveComponentRepresentative) object).getFcInterface(mc.getComponentMetadata().getComponentInterfaceName());
                    }
                    result = this.mc.execute(target);
                    complete = true;
                    synchronized(monitor){monitor.notify();}
                } else if (object instanceof ProActiveInterface) {
                    result = this.mc.execute(object);
                    complete = true;
                    synchronized(monitor){monitor.notify();}
                } else {
                    Proxy lastProxy = AbstractProcessForGroup.findLastProxy(object);
                    if (lastProxy instanceof UniversalBodyProxy) {
                        objectIsLocal = ((UniversalBodyProxy) lastProxy).isLocal();
                    }
                    if (lastProxy == null) {
                        // means we are dealing with a standard Java Object 
                        result = this.mc.execute(object);
                        complete = true;
                        synchronized(monitor){monitor.notify();}
                    } else if (!objectIsLocal) {
                        /* add the return value into the result group */
                        result = ((StubObject) object).getProxy().reify(this.mc);
                        complete = true;
                        synchronized(monitor){monitor.notify();}
                    } else {
                        /* add the return value into the result group */
                        result = ((StubObject) object).getProxy().reify(this.mc.getShallowCopy());
                        complete = true;
                        synchronized(monitor){monitor.notify();}
                    }
                }
            } catch (Throwable e) {
                /* when an exception occurs, put it in the result group instead of the (unreturned) value */
                result = new Exception(e.getLocalizedMessage()); 
                complete = true;
                synchronized(monitor){monitor.notify();}
            }
        } else {
            /* when there is a Throwable instead of an Object, a method call is impossible, add null to the result group */
            result = this.memberListOfResultGroup;
            complete = true;
            synchronized(monitor){monitor.notify();}
        }
        
    }

    public Object getResult()
    {
        if (!complete) {
            try {
                synchronized(monitor) { monitor.wait(); }
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        return result;
    }
}
