package gridcomp.manager.farm;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Logger;

import org.objectweb.proactive.core.util.wrapper.DoubleWrapper;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;
import gridcomp.manager.AbstractAutonomicManager;
import gridcomp.manager.AutonomicManagerAttributeController;
import gridcomp.manager.beans.ArrivalRateBean;
import gridcomp.manager.beans.CounterBean;
import gridcomp.manager.beans.DepartureRateBean;
import gridcomp.manager.beans.NumWorkerBean;
import gridcomp.manager.beans.QoSEvent;
import gridcomp.manager.beans.QoSItemBean;
import gridcomp.manager.operations.FarmAddExecutorCommand;
import gridcomp.manager.operations.FarmBalanceCommand;
import gridcomp.manager.operations.FarmGetExecutorsCommand;
import gridcomp.manager.operations.FarmRaiseViolationCommand;
import gridcomp.manager.operations.FarmSubExecutorCommand;
import gridcomp.manager.operations.ManagerCommand;
import gridcomp.manager.operations.ManagerOperation;
import gridcomp.manager.operations.ManagerSetupCommand;
import gridcomp.manager.util.MyMonitorFormatter;
import gridcomp.manager.util.RulesParser;
import gridcomp.operation.FarmMonitor;
import gridcomp.operation.FarmNumWorkers;
// FIXME: Needs refactoring, some attributed needs to be moved in AbstractAutonomicManager
public class AutonomicManager 
	extends AbstractAutonomicManager 
	implements AutonomicManagerAttributeController {
    private String managerFileLogger;
	private String contract;
	private String contractFilePath;
	private String monitoringMethodName;
	private final org.apache.log4j.Logger logger = org.apache.log4j.Logger.getLogger(gridcomp.log.Loggers.MANAGER);
	private final Logger remote = Logger.getLogger("MONITORING");
	
	// ********************
	// MA
	private QoSItemBean<Integer> bean3 = null;
	private QoSItemBean<Double> bean2 = null;
	private QoSItemBean<Double> bean1 = null;
	private QoSItemBean<Long> counterBean = new CounterBean(this,0L);
	
	 // so in this way, if i add a worker then i can remove easily without introspecting a component
    private ConcurrentLinkedQueue<String> workerNames = new ConcurrentLinkedQueue<String>();
    private HashMap<String,Double> methodsToMonitor = new HashMap<String, Double>();
    private boolean update =  false;
    private long startReconfTime = 0;
    public String getPerformanceContract() {
		return contract;
	}
	/* Every user could define an operation set. This operation set might include 
	 * all operations that they're available to the manager. For each operation i have
	 * a command associated.
	 * @see gridcomp.manager.operations.ManagerOperation */
	@Override 
	public void initAutonomicManager(){
       /* ManagerOperation is the action that it's given to the user in the contract. The user can use this action 
        * in the fire operation method.
        */  

		logger.info("AutonomicManager: initManager");
		addAutomomicOperation(ManagerOperation.ADD_EXECUTOR, new FarmAddExecutorCommand(managedAutonomicController,bac, workerNames));
		addAutomomicOperation(ManagerOperation.REMOVE_EXECUTOR, new FarmSubExecutorCommand(managedAutonomicController, workerNames));
		addAutomomicOperation(ManagerOperation.BALANCE_LOAD, new FarmBalanceCommand(managedAutonomicController));
		addAutomomicOperation(ManagerOperation.RAISE_VIOLATION, new FarmRaiseViolationCommand(managedAutonomicController,this));
		addAutomomicOperation(ManagerOperation.SETUP, new ManagerSetupCommand(this));
		addAutomomicOperation(ManagerOperation.GET_EXECUTORS, new FarmGetExecutorsCommand(managedAutonomicController));
	    try {
	        Handler remoteHandler = new FileHandler(managerFileLogger);
	        remoteHandler.setFormatter(new MyMonitorFormatter("# wall_clock_time observed_rate n_workers\n"));
	        remote.addHandler(remoteHandler);
	     
	    } catch (IOException e) {
	    	logger.warn("Cannot set manager logger. Maybe ManagerMonitorServer is down!");
	    }

		super.initAutonomicManager();
	}

 	/* This method is called when an operation should be done, the path is :
	 *  rule-engine-->fireOperation-->Manager-->this method-->operation to be done:
	 *  The use-case is the following:
	 *  1.Inside the autonomic loop, a violation occurs.
	 *  2. The rule engine launch the fireOperation method, which send a QoSEvent
	 *  3. The event has caught from this method
	 *  4. The event has decoded and its associated operation has fetched from an hashtable
	 *  5. The ManagerCommand gets executed
	 *  6. A reconfiguration process starts. */
	
	@Override
	public void sendEvent(GenericTypeWrapper<QoSEvent> ev) {
//logger.info("Starting a sendEvent");
		QoSEvent event = ev.getObject();
		ManagerOperation operation = event.getOperation();
		ManagerCommand cmd = lookupOperation(operation);
		cmd.setData(event.getData());
		
		if (cmd != null)
		{
			logger.info("Executing reconfiguration command");
			cmd.execute();
			
		//	logger.info("start:"+startReconfTime+"# end:"+System.currentTimeMillis());
		} else {
			logger.warn("Reconfiguration operation not allowed");
		}
	}



	@Override
	/*
	 * FIXME: should we add further beans?
	 */
	protected void setupQoS() {
		this.startReconfTime = System.currentTimeMillis();
		bean3 = createUpdateNumWorkersBean(bean3);
		for (String key: methodsToMonitor.keySet())
		{
			logger.info("Setting QoS information for method "+ key);

			//bean1 = createUpdateArrivalRateBean(key,bean1);
			bean2 = createUpdateDepartureBean(key,bean2); 

			double value = bean2.getValue();
		    logger.info("#method:"+key+"#time:"+System.currentTimeMillis()+"#rate:"+value+"#workers:"+bean3.getValue().intValue());
		    // This write a status file used for monitoring withe xternal tools. The file is specified in the ADL file of the manager
		    remote.info(System.currentTimeMillis()+"\t"+value+"\t"+bean3.getValue().intValue()+"\n");
			//System.out.println("#method:"+key+"#time:"+System.currentTimeMillis()+"#rate:"+value+"#workers:"+bean3.getValue().intValue());
		    //addQoSItem(bean1,update);	
			addQoSItem(bean2,update);
		}
	
			addQoSItem(counterBean,update);
			addQoSItem(bean3,update);
		
                //System.out.println("dopo bean tre");
			update = true;
	}
		private QoSItemBean<Double> createUpdateArrivalRateBean(String methodName, QoSItemBean<Double> bean)
	 {
		long sm = System.currentTimeMillis();
		GenericTypeWrapper<Object> obj = managedAutonomicController.execOperation(FarmMonitor.OPNAME, methodName.trim(), "server", FarmMonitor.ARRIVAL_RATE);
    	DoubleWrapper arrivalRate = (DoubleWrapper)obj.getObject();
    	long monitor_time = System.currentTimeMillis() - sm;
   		logger.info("Arrival Rate Value " + arrivalRate.doubleValue()+" for method "+methodName+" [overhead "+monitor_time+" ]");
		if (bean==null) {
			QoSItemBean<Double> newbean = new ArrivalRateBean(this,arrivalRate.doubleValue());
			return newbean;
		}
		else {
			bean.setValue(arrivalRate.doubleValue());
			return bean;
		}
	 }
	 private QoSItemBean<Double> createUpdateDepartureBean(String methodName,QoSItemBean<Double> bean) {
	     //logger.info("--MA--> (B) AutonomicManager -> DepartureRate"); 
	    long sm = System.currentTimeMillis();
        GenericTypeWrapper<Object> obj = managedAutonomicController.execOperation(FarmMonitor.OPNAME, methodName.trim(), "server", FarmMonitor.DEPARTURE_RATE);
	    DoubleWrapper departureRate = (DoubleWrapper)obj.getObject();
	    long monitor_time = System.currentTimeMillis() - sm;
	    //logger.info("--MA--> (E) AutonomicManager -> DepartureRate"+ departureRate.doubleValue());
	    logger.info("Departure Rate Value " + departureRate.doubleValue()+" for method "+methodName+" [overhead "+monitor_time+" ]");
	    if (bean==null) {
	    	QoSItemBean<Double> newbean = new DepartureRateBean(this,departureRate.doubleValue());
	    	return newbean;
	    }
	    else {
	    	bean.setValue(departureRate.doubleValue());
			return bean;
	 	}
	}
	
	private QoSItemBean<Integer> createUpdateNumWorkersBean(QoSItemBean<Integer> bean) {
		long sm = System.currentTimeMillis();
		Integer workers = (Integer)((GenericTypeWrapper) managedAutonomicController.execOperation(FarmNumWorkers.OPNAME)).getObject();
		long monitor_time = System.currentTimeMillis() - sm;
		logger.info("NumWorker Value " + workers.intValue()+" [overhead "+monitor_time+" ]");
		if (bean==null) {
			QoSItemBean<Integer> newbean = new NumWorkerBean(this,workers.intValue());
			return newbean;
		}
		else {
			bean.setValue(workers.intValue());
			return bean;
		}
	 }
	 
	/* This method present violation to the component user. 
	 * (non-Javadoc)
	 * @see gridcomp.AutonomicClientManager#raiseViolation(org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper)
	 */
	public GenericTypeWrapper<Object> raiseViolation(GenericTypeWrapper<Object> violationId) {
		return null;
	}

	public String getPerformanceContractFile() {
		return contractFilePath;
	}
	
	public void setPerformanceContractFile(String fName) {
		File f = new File(fName);
		
		if (f.exists())
		{
			logger.info("Reading contract from file " + fName);
			RulesParser parser = new RulesParser();
			logger.debug("Parsing Contract");
			String parsed = parser.parse(f.getAbsolutePath());
			logger.debug("Parsing Contract "+f.getAbsolutePath());
			if ((parsed!= null) && (parsed.length() > 0))
			{
				ArrayList<String> methodNames = parser.getMethodsNames();
				for (String method: methodNames)
				{
					String tmp = method.replace("\"", "");
					methodsToMonitor.put(tmp, 0.0);
				}
				logger.debug("Committing Contract");
				commitContract(parsed);
			}
		}else {
			logger.warn("File " +fName + " doesn't exist!");
		}
	}

	public String getMonitoringMethodName() {
		return monitoringMethodName;
	}

	public void setMonitoringMethodName(String name) {
		monitoringMethodName = name;
		
	}

	@Override
	public void addMethodToMonitor(String data) {
		if (!methodsToMonitor.containsKey(data))
		{
			methodsToMonitor.put(data, 0.0);
		}
	}

	@Override
	public void removeMethodToMonitor(String data) {
		methodsToMonitor.remove(data);
		
	}

	public String getLoggerFileName() {
		return this.managerFileLogger;
	}

	public void setLoggerFileName(String name) {
		this.managerFileLogger = name;
	}

	public long getAutonomicLoopCycle() {
		return sleepTime;
	}

	public void setAutonomicLoopCycle(long loop) {
	      this.sleepTime = loop;
	}

}
