package gridcomp.operation;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.proactive.api.PADeployment;
import org.objectweb.proactive.core.ProActiveException;
import org.objectweb.proactive.core.component.controller.MonitorController;
import org.objectweb.proactive.core.component.identity.ProActiveComponent;
import org.objectweb.proactive.core.descriptor.data.ProActiveDescriptor;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;
import org.objectweb.proactive.core.util.wrapper.StringMutableWrapper;

import gridcomp.AutonomicException;
import gridcomp.Constants;
import gridcomp.controller.AutonomicController;
import gridcomp.controller.KnowledgeConstants;
import gridcomp.farm.port.CollectiveInterfaceDispatcherProxy;
import gridcomp.log.Loggers;

public class FarmAddWorker extends AbstractAutonomicOperation
{    
	public static String OPNAME = "increaseParallelismDegree";
	private static final Logger logger = Logger.getLogger(Loggers.OPERATION);

	public FarmAddWorker(AutonomicController ac)
	{
		super(ac);
	}

	public void checkContext() throws AutonomicException
	{
		if (owner.getKnowledge().get(KnowledgeConstants.NUM_WORKERS) == null)
			throw new AutonomicException("The number of workers has not been set in context!");
		if (owner.getKnowledge().get(KnowledgeConstants.PROXY_MAP) == null)
			throw new AutonomicException("Colletive Interface Proxy Map not present in context!");
		if (owner.getKnowledge().get(KnowledgeConstants.PARENT) == null)
			throw new AutonomicException("The component has not been set in context!");
		logger.info("FarmAddWorker::Context successfully checked.");
	}

	// The first parameter must be the ADL file to be used to create the new component (as a String)
	// The second parameter must be the XML deployment descriptor file to be used to deploy the new component (as a String)
	// The third parameter must be a String used as the name of the new component

	@SuppressWarnings("unchecked")
	public Object exec(Object... params) throws AutonomicException
	{
		// Check the arguments of the operation
		if (params.length != 3)
			throw new AutonomicException("Wrong number of parameters (must be 3).");
		if (!((params[0] instanceof String && params[1] instanceof String && params[2] instanceof String) || 
				(params[0] instanceof String[] && params[1] instanceof String[] && params[2] instanceof String[])))
			throw new AutonomicException("Wrong type of parameters (all must be String or String[]).");

		String[] adlArray = null;
		String[] xmlArray = null;
		String[] nameArray = null;

		if(params[0] instanceof String){
			adlArray = new String[1];
			xmlArray = new String[1];
			nameArray = new String[1];
			
			adlArray[0] = (String) params[0];
			xmlArray[0] = (String) params[1];
			nameArray[0] = (String) params[2];
		}else{
			adlArray = (String[]) params[0];
			xmlArray = (String[]) params[1];
			nameArray = (String[]) params[2];
			
			if(!((adlArray.length == xmlArray.length) && (xmlArray.length == nameArray.length)))
				throw new AutonomicException("All the arrays passed as parameters must have the same size");
		
		}

		Vector<Component> newComponents = new Vector<Component>();

		try {
			// Retrieve the life cycle controller of the component
			ProActiveComponent compOwner = (ProActiveComponent) (owner.getKnowledge().get(KnowledgeConstants.PARENT));
			LifeCycleController lcc = (LifeCycleController) compOwner.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
			// Suspends component life cycle
			lcc.stopFc();
			
			// It uses for getting the number of loops only one array because all the arrays have the same size
			for(int i=0;i<adlArray.length;i++) {

				String adl = adlArray[i];
				String xml = xmlArray[i];
				String name = nameArray[i];

				// Check if the ADL file is in the classpath 
				if (getClass().getClassLoader().getResourceAsStream(adl.replace('.', '/') + ".fractal") == null)
					throw new AutonomicException("Cannot find '" + adl.replace('.', '/') + ".fractal" + "' in the classpath", null);

				// Check if the XML file is in the filesystem
				if (!(new File(xml)).exists())
					throw new AutonomicException("Cannot find '" + xml + "' in the filesystem", null);

				// Generates a new component
				Component newComponent = newComponent(adl,xml, name);
				newComponents.add(newComponent);
				// Check that the newComponent and parent have the same functional interfaces;
				Object[] pItfs = compOwner.getFcInterfaces();
				Object[] nItfs = compOwner.getFcInterfaces();
				for (Object o: nItfs) {
					Interface ni = (Interface)o;
					if (!ni.getFcItfName().endsWith("-controller") && !ni.getFcItfName().equals("component")) { 
						Interface pi = (Interface) compOwner.getFcInterface(ni.getFcItfName());
						if (!checkInterfaces(ni, pi))
							throw new AutonomicException("The new component interface [" + ni.getFcItfName() + "] has not an equivalent interface on the composite one");
					}
				}
				for (Object o: pItfs) {
					Interface pi = (Interface)o;
					if (!pi.getFcItfName().endsWith("-controller") && !pi.getFcItfName().equals("component")) { 
						Interface ni = (Interface) newComponent.getFcInterface(pi.getFcItfName());
						if (!checkInterfaces(ni, pi))
							throw new AutonomicException("The composite component interface [" + pi.getFcItfName() + "] has not an equivalent interface on the new one");
					}
				}            
				// Add the clones to the enclosing component
				ContentController cc = (ContentController) compOwner.getFcInterface(Constants.CONTENT_CONTROLLER);
				cc.addFcSubComponent(newComponent);

				Component internal = cc.getFcSubComponents()[0];
				String myname = ((NameController)newComponent.getFcInterface(Constants.NAME_CONTROLLER)).getFcName();
				String farmname = ((NameController)compOwner.getFcInterface(Constants.NAME_CONTROLLER)).getFcName();
				// If internal components are linked to some server interfaces, it binds the clone with them
				try {
					BindingController internalCompBC = (BindingController) internal.getFcInterface(Constants.BINDING_CONTROLLER);
					BindingController newCompBC      = (BindingController) newComponent.getFcInterface(Constants.BINDING_CONTROLLER);

					String[] internalCompItfs = internalCompBC.listFc();
					for(String internalCompItf : internalCompItfs){
						Interface itf = (Interface) internalCompBC.lookupFc(internalCompItf);
						newCompBC.bindFc(internalCompItf, itf);
						logger.info("FarmAddWorker:: New component's [" + myname + "] client interface [" + internalCompItf + "] bound to super component [" + farmname + "] server interface [" + itf.getFcItfName() + "]");
					}
				} catch (NoSuchInterfaceException e) {
					logger.info("FarmAddWorker:: Replicated component has no client interface");
				}

				// Binds each server interface of clone (except the controller ones) with a collective proxy interface
				Object[] compOwnerItfs = compOwner.getFcInterfaces();
				for (Object otf : compOwnerItfs) 
				{
					Interface itf = (Interface)otf;
					InterfaceType itfType = (InterfaceType) itf.getFcItfType();
					String        itfName = itf.getFcItfName(); 
					if (!itfType.isFcClientItf() && !itfName.endsWith("-controller") && !itfName.equals("component")) { 
						CollectiveInterfaceDispatcherProxy collPrx = null;
						HashMap<String, CollectiveInterfaceDispatcherProxy> map = (HashMap<String, CollectiveInterfaceDispatcherProxy>) (owner.getKnowledge().get(KnowledgeConstants.PROXY_MAP));
						collPrx = map.get(itf.getFcItfName());
						collPrx.add(newComponent);
						logger.info("FarmAddWorker:: New component's [" + myname + "] server interface [" + itf.getFcItfName() + "] bound to super component collective interface dispatcher proxy");
					}
				}

				// Monitor Initialization
				try {
					MonitorController mcc;
					mcc = (MonitorController)newComponent.getFcInterface(Constants.MONITOR_CONTROLLER);
					mcc.monitorInit();		
				} catch (NoSuchInterfaceException _) {
					// ignored
				}

			}


			// Update context
			int nw = (Integer) (owner.getKnowledge().get(KnowledgeConstants.NUM_WORKERS));
			owner.getKnowledge().put(KnowledgeConstants.NUM_WORKERS, nw + adlArray.length);

			// Restarts the lifecycle of enclosing component
			lcc.startFc();

			if(adlArray.length == 1) return new StringMutableWrapper(Fractal.getNameController(newComponents.elementAt(0)).getFcName());
			
			Vector<String> newComponentsName = new Vector<String>();
			
			for(int i=0;i<adlArray.length;i++){
				newComponentsName.add(Fractal.getNameController(newComponents.elementAt(i)).getFcName());
			}
			
			return new GenericTypeWrapper<Vector<String>>(newComponentsName); 
			
		} catch (Exception e) {
			e.printStackTrace();
			throw new AutonomicException(e);
		}
	}

	private Component newComponent(String adl, String xml, String name) throws ProActiveException, ADLException, NoSuchInterfaceException
	{
		System.setProperty("fractal.provider", "org.objectweb.proactive.core.component.Fractive");

		ProActiveDescriptor pad = PADeployment.getProactiveDescriptor(xml);
		Map<String, Object> context = new HashMap<String, Object>(1);
		context.put("deployment-descriptor", pad);

		Factory fact = org.objectweb.proactive.core.component.adl.FactoryFactory.getFactory();
		Component newComponent = (Component) fact.newComponent(adl, context);

		Fractal.getNameController(newComponent).setFcName(name);

		logger.info("FarmAddWorker:: New worker " + name + " created");
		return newComponent;
	}

	private boolean checkInterfaces(Interface i1, Interface i2)
	{
		InterfaceType it1 = (InterfaceType) i1.getFcItfType();
		InterfaceType it2 = (InterfaceType) i2.getFcItfType();

		if ( !it1.getFcItfName().equals(it2.getFcItfName())           ||
				!it1.getFcItfSignature().equals(it2.getFcItfSignature()) ||
				it1.isFcClientItf()     != it2.isFcClientItf()           ||
				it1.isFcCollectionItf() != it2.isFcCollectionItf()       ||
				it1.isFcOptionalItf()   != it2.isFcOptionalItf() ) 
			return false;
		return true;
	}
}
