package gridcomp.operation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import org.apache.log4j.Logger;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.proactive.core.component.Constants;
import org.objectweb.proactive.core.component.controller.QueueController;
import org.objectweb.proactive.core.component.identity.ProActiveComponent;
import org.objectweb.proactive.core.component.request.ComponentRequest;
import org.objectweb.proactive.core.util.wrapper.IntWrapper;

import gridcomp.AutonomicException;
import gridcomp.controller.AutonomicController;
import gridcomp.controller.KnowledgeConstants;
import gridcomp.log.Loggers;

public class FarmBalanceWorkers extends AbstractAutonomicOperation
{    
    public static String OPNAME = "balanceWorkerQueues";
    private static final Logger logger = Logger.getLogger(Loggers.OPERATION);
    
    public FarmBalanceWorkers(AutonomicController ac)
    {
        super(ac);

    }

    public void checkContext() throws AutonomicException
    {
        if (owner.getKnowledge().get(KnowledgeConstants.NUM_WORKERS) == null)
            throw new AutonomicException("The number of workers has not been set during initializeComponent()!");
        if (owner.getKnowledge().get(KnowledgeConstants.PARENT) == null)
            throw new AutonomicException("The component has not been set in context!");
        logger.info("FarmBalanceWorker:: Context successfully checked.");
    }

    public synchronized Object exec(Object... params) throws AutonomicException
    {
        int numWorkers = (Integer) (owner.getKnowledge().get(KnowledgeConstants.NUM_WORKERS)), mean = -1;
        if (numWorkers >= 1) {
            boolean stopped = false;
        	// Retrieve the life cycle controller of the component
            ProActiveComponent compOwner = (ProActiveComponent) (owner.getKnowledge().get(KnowledgeConstants.PARENT));
        	LifeCycleController lcc;
			try {
				lcc = (LifeCycleController) compOwner.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
				if (lcc.getFcState().equals(LifeCycleController.STARTED)) {
					// Suspends component life cycle
					try {
						lcc.stopFc();
						stopped = true;
					} catch (IllegalLifeCycleException ile) {
						throw new AutonomicException(ile);
					}
				}
				// Retrieves the internal components
				ContentController cc = (ContentController) compOwner.getFcInterface(Constants.CONTENT_CONTROLLER);
				Component[] internalComponents = cc.getFcSubComponents();
				// Retrieves queue controllers and calculates the queue length of each worker
				Map<Component, Integer> queueInt = new Hashtable<Component, Integer>();
				Map<Component, QueueController> queueContr = new Hashtable<Component, QueueController>();
				QueueController qc = null;
				Component worker = null;
				int sum = 0, length = 0;
				for (Component comp : internalComponents) {
					try {
						qc = (QueueController) comp.getFcInterface(Constants.REQUEST_QUEUE_CONTROLLER);
						length = qc.getSize().intValue();
						queueInt.put(comp, length);
						queueContr.put(comp, qc);
						sum += length;
					} catch (NoSuchInterfaceException e) {
						throw new AutonomicException("No queue controller in worker component!");
					}
				}
				mean = sum / internalComponents.length;
				List<Entry<Component, Integer>> entries = new ArrayList<Entry<Component, Integer>>(queueInt.entrySet());
				// This comparator should reorder the entries in descending order of queue length...
				Collections.sort(entries,
						new Comparator<Entry<Component, Integer>>() {
							public int compare(Entry<Component, Integer> e1, Entry<Component, Integer> e2) 
							{
								return (e2.getValue() - e1.getValue());
							}
						});
				// Resizing queues
				int cursor = 0;
				List<ComponentRequest> reqList = new Vector<ComponentRequest>(), tmpList;
				for (Entry<Component, Integer> e : entries) {
					worker = e.getKey();
					length = e.getValue();
					if (length > mean) {
						reqList.addAll(queueContr.get(worker).popRequests(length - mean));
					} else if (length < mean) {
						tmpList = reqList.subList(cursor, cursor + mean - length);
						for (ComponentRequest req : tmpList)
							queueContr.get(worker).pushRequest(req);
						cursor += (mean - length);
					}
				}
				// Adding isolated requests in first worker... not optimal temporary...
				if (cursor != reqList.size()) {
					tmpList = reqList.subList(cursor, reqList.size());
					for (ComponentRequest req : tmpList)
						queueContr.get(internalComponents[0]).pushRequest(req);
				}
			} catch (NoSuchInterfaceException nsie) {
				nsie.printStackTrace();
				throw new AutonomicException(nsie);
			}

            // Restarts the lifecycle of enclosing component
            if (stopped) {
                try {
                    lcc.startFc();
                    stopped = false;
                } catch (IllegalLifeCycleException ile) {
                	ile.printStackTrace();
                    throw new AutonomicException(ile);
                }
            }
        }
        return new IntWrapper(mean);
    }    
}
