package gridcomp.operation;

import java.util.Vector;

import org.apache.log4j.Logger;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;

import org.objectweb.proactive.core.component.controller.MethodStatistics;
import org.objectweb.proactive.core.component.controller.MonitorController;
import org.objectweb.proactive.core.util.wrapper.DoubleWrapper;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;

import gridcomp.AutonomicException;
import gridcomp.Constants;
import gridcomp.controller.AutonomicController;
import gridcomp.controller.KnowledgeConstants;
import gridcomp.log.Loggers;

public class FarmMonitor extends AbstractAutonomicOperation
{
    public static String OPNAME = "getRates";
    
    public static final String ARRIVAL_RATE   = "arrivalRate";
    public static final String DEPARTURE_RATE = "departureRate";
    
    private ContentController cc = null;
    private static final Logger logger = Logger.getLogger(Loggers.OPERATION);
    
    public FarmMonitor(AutonomicController ac)
    {
        super(ac);
    }
    
    public void checkContext() throws AutonomicException
    {
    	Component c = (Component)(owner.getKnowledge().get(KnowledgeConstants.PARENT));
    	try {
			cc = (ContentController) c.getFcInterface(Constants.CONTENT_CONTROLLER);
		} catch (NoSuchInterfaceException e) {
			throw new AutonomicException("The farm component has no " + Constants.CONTENT_CONTROLLER + " interface!");
		}   
        logger.info("FarmMonitor::Context successfully checked.");
    }
    @Override
    public Object exec(Object... params) throws AutonomicException
    {
    	// Check the arguments of the operation
    	if ((params == null)||(params.length != 3))
    		throw new AutonomicException("Wrong number of parameters (must be 3).");
    	if (!(params[0] instanceof String && params[1] instanceof String && params[2] instanceof String))
    		throw new AutonomicException("Wrong type of parameters (must be String).");
    	if (!(params[1].equals("client") || params[1].equals("server")))
    		throw new AutonomicException("Wrong second parameter (must be \"client\" or \"server\".");
    	if (!(params[2].equals(ARRIVAL_RATE) || params[2].equals(DEPARTURE_RATE)))
    		throw new AutonomicException("Wrong third parameter (must be \"" + ARRIVAL_RATE + "\" or \"" + DEPARTURE_RATE + "\".");

		try {
			Component comps[] = cc.getFcSubComponents();
			double res = 0.0;
			
			Vector<DoubleWrapper> dw_vect = new Vector<DoubleWrapper>();
			Vector<GenericTypeWrapper<MethodStatistics>> ms_vect = new Vector<GenericTypeWrapper<MethodStatistics>>();
			
			for (Component c: comps) {
				MonitorController mc = ((MonitorController)c.getFcInterface(Constants.MONITOR_CONTROLLER));
				//MethodStatistics mst = mc.getStatistics((String)params[0], (String)params[1]);
				ms_vect.add(mc.getStatistics((String)params[0], (String)params[1]));
			}
			
			for(GenericTypeWrapper<MethodStatistics> ms : ms_vect){
				if (params[2].equals(ARRIVAL_RATE)) {
					
					if (ms!=null)
					{
						DoubleWrapper dw = ms.getObject().getMeanArrivalRate();
						//res += dw.doubleValue();
						dw_vect.add(dw);
					}
				} else {
					
					if (ms!=null)
					{
						DoubleWrapper dw = ms.getObject().getMeanDepartureRate();
						//res += dw.doubleValue();
						dw_vect.add(dw);
					}
				}
			}
			
			for(DoubleWrapper dw : dw_vect){
				res += dw.doubleValue();
			}
			
			return new DoubleWrapper(res);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
    }

}

