package gridcomp.operation;

import java.util.HashMap;
import java.util.List;

import org.apache.log4j.Logger;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.proactive.core.component.controller.QueueController;
import org.objectweb.proactive.core.component.identity.ProActiveComponent;
import org.objectweb.proactive.core.component.request.ComponentRequest;
import org.objectweb.proactive.core.util.wrapper.StringMutableWrapper;

import gridcomp.AutonomicException;
import gridcomp.Constants;
import gridcomp.controller.AutonomicController;
import gridcomp.controller.KnowledgeConstants;
import gridcomp.farm.port.CollectiveInterfaceDispatcherProxy;
import gridcomp.log.Loggers;

public class FarmSubWorker extends AbstractAutonomicOperation
{
    public static String OPNAME = "decreaseParallelismDegree";
    private static final Logger logger = Logger.getLogger(Loggers.OPERATION);
    
    public FarmSubWorker(AutonomicController ac) 
    {
        super(ac);
    }

    public void checkContext() throws AutonomicException
    {
        if (owner.getKnowledge().get(KnowledgeConstants.NUM_WORKERS) == null)
            throw new AutonomicException("The number of workers has not been set in context!");
        if (owner.getKnowledge().get(KnowledgeConstants.PROXY_MAP) == null)
            throw new AutonomicException("Colletive Interface Proxy Map not present in context!");
        if (owner.getKnowledge().get(KnowledgeConstants.PARENT) == null)
            throw new AutonomicException("The component has not been set in context!");
        logger.info("FarmSubWorker::Context successfully checked.");
    }

    @SuppressWarnings("unchecked")
    @Override
    // The first parameter must be the name of the component to be removed (as a String)
    public synchronized Object exec(Object... params) throws AutonomicException
    {
    	ProActiveComponent compOwner = (ProActiveComponent) (owner.getKnowledge().get(KnowledgeConstants.PARENT));
    	
    	// Check the arguments of the operation
    	if (params.length != 1)
    		throw new AutonomicException("Wrong number of parameters (must be 1).");
    	if (!(params[0] instanceof String))
    		throw new AutonomicException("Wrong type of parameter (must be String).");
    	
    	String name = (String) params[0];
    	
    	// Check if we have at least two inner components and one with the provided name
    	int numWorkers = (Integer) (owner.getKnowledge().get(KnowledgeConstants.NUM_WORKERS));
    	if (numWorkers == 1)
    		throw new AutonomicException("Cannot remove the component [" + name +"] because there is only one component.");
    
    	Component internal = null;
    	ContentController cc = null;
    	Component internalComponents[] = null;
    	// Check if we have at least one component with the provided name
		try {
			cc = ((ContentController)compOwner.getFcInterface(Constants.CONTENT_CONTROLLER));
			internalComponents = cc.getFcSubComponents();
			for (Component c: internalComponents)
				if (Fractal.getNameController(c).getFcName().equals(name)) {
					internal = c;
					break;
				}
			if (internal == null)
				throw new AutonomicException("Cannot remove the component [" + name +"] because it does not exist.");
		} catch (NoSuchInterfaceException e) {
            e.printStackTrace();
            throw new AutonomicException(e);
		}

    	
        try {            	
        	// Retrieve the life cycle controller of the component
        	LifeCycleController lcc = (LifeCycleController) compOwner.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
        	// Suspends component lifecycle
        	lcc.stopFc();
        	// Retrieve the qc to redistribute request calls
        	QueueController qc = (QueueController) internal.getFcInterface(Constants.REQUEST_QUEUE_CONTROLLER);
        	int length = qc.getSize().intValue();
        	// Retrieves the functional component requests from the component to be removed
        	List<ComponentRequest> list = qc.popRequests(length);  
        	// Retrieves the interfaces of composite component
            Object[] ownerItfs = compOwner.getFcInterfaces();
            // unbind the component to be removed from all the collective interfaces
            for (Object otf : ownerItfs) {
            	Interface itf = (Interface)otf;
            	InterfaceType itfType = (InterfaceType) itf.getFcItfType();
            	String        itfName = itf.getFcItfName(); 
            	if (!itfType.isFcClientItf() && !itfName.endsWith("-controller") && !itfName.equals("component")) { 
            		CollectiveInterfaceDispatcherProxy collPrx = null;
            		HashMap<String, CollectiveInterfaceDispatcherProxy> map = (HashMap<String, CollectiveInterfaceDispatcherProxy>) (owner.getKnowledge().get(KnowledgeConstants.PROXY_MAP));
            		collPrx = map.get(itf.getFcItfName());
            		collPrx.remove(internal.getFcInterface(itf.getFcItfName()));
            		logger.info("FarmSubWorker:: Removed binding to [" + itf.getFcItfName() + "] proxy.");
            	}
            }
            // unbind the component to be removed from all the server interfaces
            BindingController bc = Fractal.getBindingController(internal);
            String[] clientItfs = bc.listFc();
            for (String itfName: clientItfs) {
            	bc.unbindFc(itfName);
            	logger.info("FarmSubWorker:: Removed binding to [" + itfName + "] server interface on parent.");
            }
            // remove the component  from the super component which contains it
            cc.removeFcSubComponent(internal);
            internalComponents = cc.getFcSubComponents();
            // add the pending component requests and one per remaining worker
            int workersNumber = internalComponents.length;
            int counter = 0;
            for (ComponentRequest req: list) {
            	qc = (QueueController) internalComponents[counter].getFcInterface(Constants.REQUEST_QUEUE_CONTROLLER);
            	qc.pushRequest(req);
            	counter = (counter + 1) % workersNumber;
            }
            // Restarts the lifecycle of the enclosing component
            lcc.startFc();
            // Update context
            owner.getKnowledge().put(KnowledgeConstants.NUM_WORKERS, workersNumber);
            
            return new StringMutableWrapper(Fractal.getNameController(internal).getFcName());
        } catch (Exception e) {
            e.printStackTrace();
            throw new AutonomicException(e);
        }
    }
}
