package gridcomp.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import gridcomp.Constants;
import gridcomp.controller.AutonomicController;
import gridcomp.controller.BeskeAttributeController;
import gridcomp.log.Loggers;
import gridcomp.operation.FarmAddWorker;
import gridcomp.operation.FarmBalanceWorkers;
import gridcomp.operation.FarmNumWorkers;
import gridcomp.operation.FarmSubWorker;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.junit.Before;
import org.junit.Test;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.proactive.api.PADeployment;
import org.objectweb.proactive.core.component.controller.QueueController;
import org.objectweb.proactive.core.descriptor.data.ProActiveDescriptor;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;

public class BalancingFarmControllerTest extends GridcompTest
{
	public static final String NAME_PREFIX = "new_";
	public static final int    NUM_CALLS = 60;
	
	protected Component root = null;
	protected Runner run = null;
	protected Getter get = null;
	protected AutonomicController abc = null;
	protected BeskeAttributeController bac = null;
	protected ContentController fcc = null;
    
    public static final Logger logger = Logger.getLogger(Loggers.CONTROLLER_TEST);
    
    @Before
    public void createComponents() throws Exception
    {
    	
        ProActiveDescriptor pad = PADeployment.getProactiveDescriptor("local-deployment.xml");
        Map<String, Object> context = new HashMap<String, Object>(1);
        context.put("deployment-descriptor", pad);

        Factory fact = gridcomp.adl.FactoryFactory.getFactory();
        root =  (Component) fact.newComponent("gridcomp.test.adl.BalancingFarmControllerTest", context);
        
        assertNotNull(root);
                
        ContentController cc = Fractal.getContentController(root);
        Component[] comps = cc.getFcSubComponents();
        for (Component c: comps) {
            if (Fractal.getNameController(c).getFcName().equals("farm")) {
                abc = (AutonomicController) c.getFcInterface(Constants.AUTONOMIC_CONTROLLER);
                bac = (BeskeAttributeController) c.getFcInterface(Constants.BESKE_ATTRIBUTE_CONTROLLER);
                fcc = (ContentController) c.getFcInterface(Constants.CONTENT_CONTROLLER);
            } else if (Fractal.getNameController(c).getFcName().equals("master")) {
            	run = (Runner) c.getFcInterface("run");
            	get = (Getter) c.getFcInterface("get");
            }

        }
        
        assertNotNull(run);
        assertNotNull(get);
        assertNotNull(abc);
        assertNotNull(bac);
        assertNotNull(fcc);
    }
    
    protected int num()
    {
    	GenericTypeWrapper<Object> obj = abc.execOperation(FarmNumWorkers.OPNAME);
    	return Integer.parseInt(obj.getObject().toString());    	
    }
    
    protected void add(int id)
    {
    	String adl = bac.getProperty("adl.dir").replace('/', '.') + ".new_worker_" + id;
    	String xml = bac.getProperty("xml.file");
    	abc.execOperation(FarmAddWorker.OPNAME, adl, xml, NAME_PREFIX + id);
    }
    
    protected void sub(int id)
    {
    	abc.execOperation(FarmSubWorker.OPNAME, NAME_PREFIX + id);
    }

    protected void balance()
    {
    	abc.execOperation(FarmBalanceWorkers.OPNAME);
    }	
    
    @Test
    public void FarmNumWorkers()
    {
    	assertEquals(num(), 1);
    }
    
    @Test
    public void FarmAddWorker()
    {
    	add(1);
    	assertEquals(num(), 2);
    	add(2);
    	assertEquals(num(), 3);
    	add(3);
    	assertEquals(num(), 4);
    }

    @Test
    public void FarmSubWorker()
    {
    	add(1);
    	add(2);
    	sub(1);
    	assertEquals(num(), 2);
    	sub(2);
    	assertEquals(num(), 1);
    }
    
    @Test
    public void FarmBalanceWorkers() throws Exception
    {	
    	add(1);
        balance();
    }

    @Test
    public void ExecutionTest() throws Exception
    {
    	LifeCycleController lcc = (LifeCycleController) root.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
    	Component[] compsf = null;
    	
        int res = 0;

        lcc.startFc();

        for (int i = 0; i < NUM_CALLS; i++)
            run.run();

        Thread.sleep(3000);
        
        add(1);
        balance();
        
        compsf = fcc.getFcSubComponents();
        for (Component cf: compsf) {
        	QueueController qc = (QueueController) cf.getFcInterface(Constants.REQUEST_QUEUE_CONTROLLER);
        	NameController nc = (NameController) cf.getFcInterface(Constants.NAME_CONTROLLER);
        	int size = qc.getSize().intValue();
        	logger.info("Worker " + nc.getFcName() + " has " + size + " calls enqueued");
        }

        Thread.sleep(3000);
        
        sub(1);
        
        compsf = fcc.getFcSubComponents();
        for (Component cf: compsf) {
        	QueueController qc = (QueueController) cf.getFcInterface(Constants.REQUEST_QUEUE_CONTROLLER);
        	NameController nc = (NameController) cf.getFcInterface(Constants.NAME_CONTROLLER);
        	int size = qc.getSize().intValue();
        	logger.info("Worker " + nc.getFcName() + " has " + size + " calls enqueued");
        }

        Thread.sleep(3000);
        
        add(2);
        add(3);
        balance();
        
        compsf = fcc.getFcSubComponents();
        for (Component cf: compsf) {
        	QueueController qc = (QueueController) cf.getFcInterface(Constants.REQUEST_QUEUE_CONTROLLER);
        	NameController nc = (NameController) cf.getFcInterface(Constants.NAME_CONTROLLER);
        	int size = qc.getSize().intValue();
        	logger.info("Worker " + nc.getFcName() + " has " + size + " calls enqueued");
        }
                    	
        // Active wait... no other workaround possible!
        res = get.get().intValue();
		logger.info("MasterImpl::test::received " + res + " signals");
        while (res < NUM_CALLS) {
    		try { Thread.sleep(1000); } catch (InterruptedException _) {}
    		res = get.get().intValue();
    		logger.info("MasterImpl::test::received " + res + " signals");
        }

        System.out.println("All the " + (NUM_CALLS) + " calls have been successfully executed!");
        
        lcc.stopFc();

    }
}
