package gridcomp.test;

import static org.junit.Assert.*;

import gridcomp.Constants;
import gridcomp.controller.AutonomicController;
import gridcomp.controller.BeskeAttributeController;
import gridcomp.log.Loggers;
import gridcomp.operation.FarmAddWorker;
import gridcomp.operation.FarmBalanceWorkers;
import gridcomp.operation.FarmNumWorkers;
import gridcomp.operation.FarmSubWorker;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.junit.*;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.proactive.api.PADeployment;
import org.objectweb.proactive.core.descriptor.data.ProActiveDescriptor;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;

public class FarmControllerTest extends GridcompTest
{
	public static final String NAME_PREFIX = "new_";
	public static final int    NUM_CALLS = 1;
	
	protected Component root = null;
	protected Runner run = null;
	protected Getter get = null;
	protected AutonomicController abc = null;
	protected BeskeAttributeController bac = null;
    
    public static final Logger logger = Logger.getLogger(Loggers.CONTROLLER_TEST);
    
    @Before
    public void createComponents() throws Exception
    {
        ProActiveDescriptor pad = PADeployment.getProactiveDescriptor("local-deployment.xml");
        Map<String, Object> context = new HashMap<String, Object>(1);
        context.put("deployment-descriptor", pad);

        Factory fact = gridcomp.adl.FactoryFactory.getFactory();
        root =  (Component) fact.newComponent("gridcomp.test.adl.FarmControllerTest", context);
        
        assertNotNull(root);
                
        ContentController cc = Fractal.getContentController(root);
        Component[] comps = cc.getFcSubComponents();
        for (Component c: comps) {
            if (Fractal.getNameController(c).getFcName().equals("farm")) {
                abc = (AutonomicController) c.getFcInterface(Constants.AUTONOMIC_CONTROLLER);
                bac = (BeskeAttributeController) c.getFcInterface(Constants.BESKE_ATTRIBUTE_CONTROLLER);
            } else if (Fractal.getNameController(c).getFcName().equals("master")) {
            	run = (Runner) c.getFcInterface("run");
            	get = (Getter) c.getFcInterface("get");
            }

        }
        assertNotNull(run);
        assertNotNull(get);
        assertNotNull(abc);
        assertNotNull(bac);
    }
    
    protected int num()
    {
    	GenericTypeWrapper<Object> obj = abc.execOperation(FarmNumWorkers.OPNAME);
    	return Integer.parseInt(obj.getObject().toString());    	
    }
    
    protected void add(int id)
    {
    	String adl = bac.getProperty("adl.dir").replace('/', '.') + ".new_worker_" + id;
    	String xml = bac.getProperty("xml.file");
    	abc.execOperation(FarmAddWorker.OPNAME, adl, xml, NAME_PREFIX + id);
    }
    
    protected void sub(int id)
    {
    	abc.execOperation(FarmSubWorker.OPNAME, NAME_PREFIX + id);
    }

    protected void balance()
    {
    	abc.execOperation(FarmBalanceWorkers.OPNAME);
    }

    @Test
    public void FarmNumWorkers()
    {
    	assertEquals(num(), 1);
    }
    
    @Test
    public void FarmAddWorker()
    {
    	add(1);
    	assertEquals(num(), 2);
    	add(2);
    	assertEquals(num(), 3);
    	add(3);
    	assertEquals(num(), 4);
    }

    @Test
    public void FarmSubWorker()
    {
    	sub(1);
    }

    @Test
    public void FarmBalanceWorkers()
    {
    	balance();
    }

    @Test
    public void ExecutionTest() throws Exception
    {
        Fractal.getLifeCycleController(root).startFc();

        for (int i = 0; i < NUM_CALLS; i++)
            run.run();

        // Active wait... no other workaround possible!
        int res = get.get().intValue();
        while (res != NUM_CALLS) {
    		try { Thread.sleep(1000); } catch (InterruptedException _) {}
    		res = get.get().intValue();
        }
        System.out.println("All " + NUM_CALLS + " calls have been successfully executed on the first pass!");
        
        add(1);
        
        for (int i = 0; i < NUM_CALLS; i++)
            run.run();

        // Active wait... no other workaround possible!
        res = get.get().intValue();
        while (res != 2 * NUM_CALLS) {
    		try { Thread.sleep(1000); } catch (InterruptedException _) {}
    		res = get.get().intValue();
        }
        System.out.println("All " + NUM_CALLS + " calls have been successfully executed on the second pass!");

        add(2);
        
        for (int i = 0; i < NUM_CALLS; i++)
            run.run();

        // Active wait... no other workaround possible!
        res = get.get().intValue();
        while (res != 3 * NUM_CALLS) {
    		try { Thread.sleep(1000); } catch (InterruptedException _) {}
    		res = get.get().intValue();
        }
        System.out.println("All " + NUM_CALLS + " calls have been successfully executed on the third pass!");

        add(3);
        
        for (int i = 0; i < NUM_CALLS; i++)
            run.run();

        // Active wait... no other workaround possible!
        res = get.get().intValue();
        while (res != 3 * NUM_CALLS) {
    		try { Thread.sleep(1000); } catch (InterruptedException _) {}
    		res = get.get().intValue();
        }
        System.out.println("All " + NUM_CALLS + " calls have been successfully executed on the fourth pass!");

        Fractal.getLifeCycleController(root).stopFc();
    }
}
