package gridcomp.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import gridcomp.Constants;
import gridcomp.controller.AutonomicController;
import gridcomp.controller.BeskeAttributeController;
import gridcomp.log.Loggers;
import gridcomp.operation.FarmAddWorker;
import gridcomp.operation.FarmBalanceWorkers;
import gridcomp.operation.FarmMonitor;
import gridcomp.operation.FarmNumWorkers;
import gridcomp.operation.FarmSubWorker;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.junit.Before;
import org.junit.Test;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.proactive.api.PADeployment;
import org.objectweb.proactive.core.descriptor.data.ProActiveDescriptor;
import org.objectweb.proactive.core.util.wrapper.GenericTypeWrapper;

public class MonitorBalanceFarmControllerTest extends GridcompTest
{
	public static final String NAME_PREFIX = "new_";
	public static final int    NUM_CALLS = 24;
	
	protected Component root = null;
	protected Runner run = null;
	protected Getter get = null;
	protected AutonomicController      abc = null;
	protected BeskeAttributeController bac = null;
	protected ContentController        fcc = null;
    
    public static final Logger logger = Logger.getLogger(Loggers.CONTROLLER_TEST);
    
    @Before
    public void createComponents() throws Exception
    {
        ProActiveDescriptor pad = PADeployment.getProactiveDescriptor("local-deployment.xml");
        Map<String, Object> context = new HashMap<String, Object>(1);
        context.put("deployment-descriptor", pad);

        Factory fact = gridcomp.adl.FactoryFactory.getFactory();
        root =  (Component) fact.newComponent("gridcomp.test.adl.MonitorBalanceFarmControllerTest", context);
        
        assertNotNull(root);
                
        ContentController cc = Fractal.getContentController(root);
        Component[] comps = cc.getFcSubComponents();
        for (Component c: comps) {
            if (Fractal.getNameController(c).getFcName().equals("farm")) {
                abc = (AutonomicController) c.getFcInterface(Constants.AUTONOMIC_CONTROLLER);
                bac = (BeskeAttributeController) c.getFcInterface(Constants.BESKE_ATTRIBUTE_CONTROLLER);
                fcc = (ContentController) c.getFcInterface(Constants.CONTENT_CONTROLLER);
            } else if (Fractal.getNameController(c).getFcName().equals("master")) {
            	run = (Runner) c.getFcInterface("run");
            	get = (Getter) c.getFcInterface("get");
            }

        }
        assertNotNull(run);
        assertNotNull(get);
        assertNotNull(abc);
        assertNotNull(bac);
        assertNotNull(fcc);
    }
    
    protected int num()
    {
    	GenericTypeWrapper<Object> obj = abc.execOperation(FarmNumWorkers.OPNAME);
    	return Integer.parseInt(obj.getObject().toString());    	
    }
    
    protected void add(int id)
    {
    	String adl = bac.getProperty("adl.dir").replace('/', '.') + ".new_worker_" + id;
    	String xml = bac.getProperty("xml.file");
    	abc.execOperation(FarmAddWorker.OPNAME, adl, xml, NAME_PREFIX + id);
    }
    
    protected void sub(int id)
    {
    	abc.execOperation(FarmSubWorker.OPNAME, NAME_PREFIX + id);
    }

    protected void balance()
    {
    	abc.execOperation(FarmBalanceWorkers.OPNAME);
    }	
    
	protected double monitor(String type)
    {
    	GenericTypeWrapper<Object> obj = abc.execOperation(FarmMonitor.OPNAME, "voidCall", "server", type);
    	return Double.parseDouble(obj.getObject().toString());    	
    }

	
    protected void show(List<Map<String, Double>> d)
    {
    	Map<String, Double> deps = d.get(0);
    	Map<String, Double> arrs = d.get(1);
    	for (String n: deps.keySet())
    		System.err.println(n + ": Dep Rate = " + deps.get(n) + ", Arr Rate = " + arrs.get(n));
    }
            
    @Test
    public void FarmMonitor() throws Exception
    {	
    	double dep = monitor(FarmMonitor.DEPARTURE_RATE);
    	double arr = monitor(FarmMonitor.ARRIVAL_RATE);
		System.err.println("Dep Rate = " + dep + " calls/sec, Arr Rate = " + arr + " calls/sec");

		assertEquals(dep, 0.0, 0.01);
		assertEquals(arr, 0.0, 0.01);
    }

    @Test
    public void ExecutionTest1() throws Exception
    {
    	LifeCycleController lcc = (LifeCycleController) root.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
    	
        int res = 0;

        lcc.startFc();

        for (int i = 0; i < NUM_CALLS; i++)
            run.run();

        Thread.sleep((NUM_CALLS+1)*1000);
    	double dep = monitor(FarmMonitor.DEPARTURE_RATE);
    	double arr = monitor(FarmMonitor.ARRIVAL_RATE);
		System.err.println("Dep Rate = " + dep + " calls/sec, Arr Rate = " + arr + " calls/sec");

		assertEquals(dep, 0.9, 0.1);
 
        // Active wait... no other workaround possible!
        res = get.get().intValue();
		logger.info("MasterImpl::test::received " + res + " signals");
        while (res < NUM_CALLS) {
    		try { Thread.sleep(1000); } catch (InterruptedException _) {}
    		res = get.get().intValue();
    		logger.info("MasterImpl::test::received " + res + " signals");
        }

        System.out.println("All the " + (NUM_CALLS) + " calls have been successfully executed!");
        
        lcc.stopFc();

    }
    

    @Test
    public void ExecutionTest2() throws Exception
    {
    	LifeCycleController lcc = (LifeCycleController) root.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
    	
        int res = 0;

        lcc.startFc();
        add(1);

        for (int i = 0; i < NUM_CALLS; i++)
            run.run();

        Thread.sleep((NUM_CALLS+1)*1000);
    	double dep = monitor(FarmMonitor.DEPARTURE_RATE);
    	double arr = monitor(FarmMonitor.ARRIVAL_RATE);
		System.err.println("Dep Rate = " + dep + " calls/sec, Arr Rate = " + arr + " calls/sec");

		assertEquals(dep, 1.8, 0.2);

        // Active wait... no other workaround possible!
        res = get.get().intValue();
		logger.info("MasterImpl::test::received " + res + " signals");
        while (res < NUM_CALLS) {
    		try { Thread.sleep(1000); } catch (InterruptedException _) {}
    		res = get.get().intValue();
    		logger.info("MasterImpl::test::received " + res + " signals");
        }

        System.out.println("All the " + (NUM_CALLS) + " calls have been successfully executed!");
        
        lcc.stopFc();

    }

    @Test
    public void ExecutionTest3() throws Exception
    {
    	LifeCycleController lcc = (LifeCycleController) root.getFcInterface(Constants.LIFECYCLE_CONTROLLER);
    	
        int res = 0;

        lcc.startFc();
        add(1);
        add(2);

        for (int i = 0; i < NUM_CALLS; i++)
            run.run();

        Thread.sleep((NUM_CALLS+1)*1000);
    	double dep = monitor(FarmMonitor.DEPARTURE_RATE);
    	double arr = monitor(FarmMonitor.ARRIVAL_RATE);
		System.err.println("Dep Rate = " + dep + " calls/sec, Arr Rate = " + arr + " calls/sec");

		assertEquals(dep, 2.7, 0.3);
 
        // Active wait... no other workaround possible!
        res = get.get().intValue();
		logger.info("MasterImpl::test::received " + res + " signals");
        while (res < NUM_CALLS) {
    		try { Thread.sleep(1000); } catch (InterruptedException _) {}
    		res = get.get().intValue();
    		logger.info("MasterImpl::test::received " + res + " signals");
        }

        System.out.println("All the " + (NUM_CALLS) + " calls have been successfully executed!");
        
        lcc.stopFc();

    }
}
