package gridcomp.manager.gui;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;

import javax.swing.*;
import org.jfree.chart.*;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.title.TextTitle;
import org.jfree.data.xy.*;
import org.jfree.ui.RefineryUtilities;
import org.jfree.data.DomainOrder;
import java.util.Random;
 
public class ManagerMonitorFrame extends JFrame implements ManagerMonitorListener
{
    private static final String TITLE = "Behavioural Skeleton Monitor";
	private ManagerMonitorModel model;
	private RateXYDataset set;
	private WorkerXYDataset workers;
	private int currentWorkers = 1;
	JFreeChart jfreechart;
	boolean firstTime = true;
	boolean firstWorkerTime = true;
	private double thresold = 3.0;
	public ManagerMonitorFrame(String title, ManagerMonitorModel m, double thresold)
    {
        super(title);
        model = m;
        this.model.addModelListener(this);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        set = RateXYDataset.createDataSet(thresold);
        workers = WorkerXYDataset.createDataSet();
        
        jfreechart = createChart(set);
        JFreeChart chart1 = createChartWorker(workers);
		//jfreechart.addSubtitle(0, subtitle);
        ChartPanel  chartPanel_grandeTaille = new ChartPanel(jfreechart);
        ChartPanel  chartPanel_grandeTaille1 = new ChartPanel(chart1);
        
        JPanel panel = new JPanel();
        
        Dimension grandeTaille = new Dimension(400, 380);
        chartPanel_grandeTaille.setMaximumSize(grandeTaille);
        chartPanel_grandeTaille.setPreferredSize(grandeTaille);
        chartPanel_grandeTaille.setMinimumSize(grandeTaille);
        Dimension grandeTaille1 = new Dimension(400, 380);
        chartPanel_grandeTaille1.setMaximumSize(grandeTaille);
        chartPanel_grandeTaille1.setPreferredSize(grandeTaille);
        chartPanel_grandeTaille1.setMinimumSize(grandeTaille);
       
        panel.setLocation(0,0);
        panel.setLayout(new FlowLayout(FlowLayout.LEFT));
        panel.setPreferredSize(new Dimension(1000, 420));
        
        panel.add(chartPanel_grandeTaille);
        panel.add(chartPanel_grandeTaille1);
        setContentPane( panel);
    }
	
    
    private static JFreeChart createChart(XYDataset xydataset)
    
    {
    //	ChartFactory.c
        JFreeChart jfreechart = ChartFactory.createXYLineChart("Throughput Monitor", "Time", "Throughput", xydataset, PlotOrientation.VERTICAL, true, true, false);
        return jfreechart;
    }
    private static JFreeChart createChartWorker(XYDataset xydataset)
    
    {
    //	ChartFactory.c
        JFreeChart jfreechart = ChartFactory.createXYLineChart("Worker Monitor", "Time", "#Workers", xydataset, PlotOrientation.VERTICAL, true, true, false);
        return jfreechart;
    }
    
    
    
    /*
    public static JPanel createDemoPanel()
    {
        JFreeChart jfreechart = createChart(set);
        return new ChartPanel(jfreechart);
    }
    */
    
  /*  public static void main(String args[])
    {
        ManagerMonitorFrame xyseriesdemo1a = new ManagerMonitorFrame(TITLE,null);
        xyseriesdemo1a.pack();
        RefineryUtilities.centerFrameOnScreen(xyseriesdemo1a);
        xyseriesdemo1a.setVisible(true);
    }*/

	public void update(ManagerMonitorEvent ev) {
		// TODO Auto-generated method stub
		double rate = ev.getRate();
		double time = ev.getTime();
		
		System.out.println("Updating GUI with value " + rate);
		
		double[] xval = set.getXValues();
		if (firstTime){
			xval[0] = ev.getTime();
		    firstTime = false;		
		}
		double[] yval = set.getYValues();
		double[] xval_new = new double[xval.length+1];
		double[] yval_new = new double[yval.length+1];
		
		System.arraycopy(xval,0,xval_new, 0, xval.length);
		System.arraycopy(yval,0,yval_new, 0, yval.length);
		
		xval_new[xval.length] = time;
		
		yval_new[yval.length] = rate;
		set.updateDataSet(xval_new, yval_new);
		
		double[] worker_xval = workers.getXValues();
		if (firstWorkerTime){
			worker_xval[0] = ev.getTime();
		    firstWorkerTime = false;		
		}
		double[] worker_yval = workers.getYValues();
		double[] worker_xval_new = new double[worker_xval.length+1];
		double[] worker_yval_new = new double[worker_yval.length+1];
		
		System.arraycopy(worker_xval,0,worker_xval_new, 0, worker_xval.length);
		System.arraycopy(worker_yval,0,worker_yval_new, 0, worker_yval.length);
		
		
		if ((ev.getWorkers()>0) && (ev.getTime()>0))
		{
		worker_xval_new[xval.length] = ev.getTime();
		worker_yval_new[yval.length] = ev.getWorkers();
		workers.updateDataSet(worker_xval_new, worker_yval_new);
		}
		xval = null;
		yval = null;
	}
}

